#!/usr/bin/env python
"""A class to perform linear interpolation on data read from csv file."""

# Copyright 2010 Kevin Keating
# 
# Licensed under the Educational Community License, Version 2.0 (the
# "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at
# 
# http://www.osedu.org/licenses/ECL-2.0
# 
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.

from math import floor

class LinearInterp:
    """A class to perform linear interpolation on data read from csv file."""
    
    def __init__(self, fileName):
        """Initialize the class and read data from the specified a file name.
        
        ARGUMENTS:
            fileName - the name of the CSV data file.
                       This CSV file should have two columns: x and y, as well as a header row (which will be ignored).
        RETURNS:
            an initialized LinearInterp object
        """
        
        self.__xStart   = None #the starting value for the x-axis
        self.__xStep    = None #the size of the increment along the x-axis
        self.__fileName = None #the input filename
        self.__yData    = []   #the y values read from fileName
        self.__fileName = fileName
        
        input = open(fileName, 'r')
        input.readline() #skip the header
        
        #read in the first line, which determines xStart
        curdata = input.readline().split(",")
        self.__xStart = float(curdata[0])
        self.__yData.append(float(curdata[1]))
        
        #read in the second line, which determines xStep
        curdata = input.readline().split(",")
        self.__xStep = float(curdata[0]) - float(self.__xStart)
        self.__yData.append(float(curdata[1]))
        
        ep = self.__xStep / 10000 #since the step size is a float, we can't test that each step size is exactly right
                                #instead, we make sure that the step size is within .01% of the correct size
                                #using this epsilon value
        
        #read in the rest of the data file
        for curline in input:
            
            #skip blank lines
            if len(curline.rstrip()) == 0:
                continue
            curdata = curline.split(",")
            
            #make sure that step size is correct
            xVal = self.__xStart + self.__xStep * len(self.__yData)
            if (xVal - ep <= curdata[0]) and (curdata[0] <= xVal + ep):
                print "Incorrect step size in file " + self.__fileName
            
            self.__yData.append(float(curdata[1]))
        input.close()
    
    
    def interp(self, xVal):
        """Determine the interpolated (or extrapolated) y value for the given x value
        
        ARGUMENTS:
            xVal - the x value to determine the y value for
        RETURNS:
            yVal - the interpolated (or extrapolated) y value
        """
        
        xPos = int(floor( (xVal-self.__xStart) / self.__xStep ))
        
        #determine if this is an interpolation or an extrapolation
        yVal = None
        if xPos < 0:
            #extrapolation to lower x values
            yStep = self.__yData[0] - self.__yData[1]
            numSteps = (xVal-self.__xStart) / self.__xStep 
            yVal = self.__yData[0] - numSteps * yStep
            #print "Extrapolating to lower x value (" + self.__fileName + "): (%(xVal)f, %(yVal)f)" %vars()
        elif xPos > len(self.__yData)-1:
            #extrapolation to higher x values
            yStep = self.__yData[-1] - self.__yData[-2]
            numSteps = (xVal-self.__xStart) / self.__xStep - (len(self.__yData) - 1)
            yVal = self.__yData[-1] + numSteps * yStep
            #print "Extrapolating to higher x value (" + self.__fileName + "): (%(xVal)f, %(yVal)f)" %vars()
        else:
            #interpolation
            remainder = xVal - self.__xStart - self.__xStep * xPos
            yVal = self.__yData[xPos] * ( 1 - remainder/self.__xStep) + self.__yData[xPos+1] * (remainder/self.__xStep)
            #print "Interpolating (" + self.__fileName + "): (%(xVal)f, %(yVal)g)" %vars()
        return yVal




# coding: utf-8
from __future__ import print_function
import pyrogen
import pyrogen_boost
import atom_types
import rdkit
from rdkit import Chem
from rdkit.Chem import AllChem
import rdkit.Chem.Draw
from rdkit.Chem.Draw import IPythonConsole
import enum
from enum import Enum

# for dictionaries
import pyrogen_swig as pysw

def write_mols(suppl):
    for idx,mol in enumerate(suppl):
        fn = 'pf-'+str(idx)+'.sdf'
        # print('writing {}'.format(fn))
        print(Chem.MolToMolBlock(mol), file=file(fn, 'w'))

# return True or Less or Greater or Confused_T_1 or Confused_T_2
match_type = Enum('match', 'MATCH ABOVE BELOW CONFUSED_T1A CONFUSED_T1B CONFUSED_T2A CONFUSED_T2B UNKNOWN WRONG_ELEMENT')

# for example CT, CJ returns BELOW because CT (check_coot_amber_type) is below CJ (ref_type)
#
class compare_types:

    def check(self, check_coot_amber_type, ref_type, Cs):
        if check_coot_amber_type == ref_type:
            return match_type.MATCH
        if not check_coot_amber_type in Cs:
            return match_type.UNKNOWN
        rank_c_forward  = self.get_rank(check_coot_amber_type, Cs)
        rank_r_forward  = self.get_rank(ref_type, Cs)
        rank_c_backward = self.get_rank(check_coot_amber_type, Cs, reverse_search=True)
        rank_r_backward = self.get_rank(ref_type, Cs, reverse_search=True)
        name = "####"
        try:
            name = self.atom.GetProp('name')
        except 'green' as e:
            print('no name for {}'.format(self.atom))
        s = "{}: {} should be {} in {} "
        s += "rank_c_forward {} rank_r_forward {} rank_c_backward {} rank_r_backward {}"
        if self.debug_output:
           print(s.format(name, check_coot_amber_type, ref_type, Cs, rank_c_forward, rank_r_forward, rank_c_backward, rank_r_backward))
        if rank_c_forward < rank_r_forward:
            if rank_c_backward < rank_r_backward:
                return match_type.ABOVE
        if rank_c_forward > rank_r_forward:
            if rank_c_backward > rank_r_backward:
                return match_type.BELOW
        # there is some type of confusion
        if rank_c_forward < rank_r_forward:
            if rank_c_backward < rank_r_backward:
                return match_type.CONFUSED_T1A
            else:
                return match_type.CONFUSED_T1B
        if rank_c_forward > rank_r_forward:
            if rank_c_backward < rank_r_backward:
                return match_type.CONFUSED_T2A
            else:
                return match_type.CONFUSED_T1B
        return match_type.UNKNOWN

    def get_rank(self, a_type, types, reverse_search=False):
        if not reverse_search:
            for i,t in enumerate(types):
                if t == a_type:
                    return i
            print('compare_types() error')  # shouldn't happen
            return 'rank-not-found'
        else:
            n = len(types)
            for i in range(n-1,-1,-1):
               if a_type == types[i]:
                  return i
        print("error in get_rank()")
        return -1

    def test_compare_2(self):

       Ns = ['NC', 'N2', 'NB', 'N', 'N2', 'N']
       # Ns = ['NC', 'N2', 'NC']
       for N_1 in Ns:
          r1 = self.get_rank(N_1, Ns)
          print("type {:>2} rank {:>2}".format(N_1, r1))

       N_1 = 'NB'
       N_2 = 'N2'
       r = self.check(N_1, N_1, Ns)
       print("{:>2} and {:>2} {:>2}".format(N_1, N_1, r))
       r = self.check(N_2, N_2, Ns)
       print("{:>2} and {:>2} {:>2}".format(N_2, N_2, r))
       r = self.check(N_1, N_2, Ns)
       print("{:>2} and {:>2} {:>2}".format(N_1, N_2, r))
       r = self.check(N_2, N_1, Ns)
       print("{:>2} and {:>2} {:>2}".format(N_2, N_1, r))

    def compare(self, check_coot_amber_type, ref_type):

       if check_coot_amber_type == ref_type:
           return match_type.MATCH
       ele = ref_type[0];
       if ele == 'C':
           r = self.check(check_coot_amber_type, ref_type, self.Cs)
           return r
       else:
           if ele == 'N':
               r = self.check(check_coot_amber_type, ref_type, self.Ns)
               return r
           else:
               if ele == 'O':
                   r = self.check(check_coot_amber_type, ref_type, self.Os)
                   return r
               else:
                   if ele == 'S':
                       r = self.check(check_coot_amber_type, ref_type, self.Ss)
                       return r
                   else:
                       return match_type.WRONG_ELEMENT # unknown element


    def __init__(self, atom_in, debug_output=False):

       self.debug_output = False
       if debug_output:
           self.debug_output = True
       self.atom = atom_in
       self.Cs = ['CB', 'CR', 'CB', 'CW', 'CC', 'CR', 'CW', 'CC', 'CR', 'C', 'CP', 'C',
                  'CR', 'CB', 'C*', 'CA', 'CM', 'C2', 'CJ', 'CT']
       self.Ns = ['NB', 'NJ', 'NL', 'N3',  'N3', 'NC', 'N2', 'NB', 'N', 'N2', 'N',
                  'NA', 'N3', 'ND', "N2", "N", "N*", "N3", "Nu"]
       self.Os = ['OW', 'OH', 'OS', 'O2', 'OS', 'O2', 'O', 'Ou']
       self.Ss = ['SH', 'S', 'SO', 'SD', 'Su']

       # self.test_compare_2()

def get_parmfrosst_types(m, mol_idx, ref_types=None, check_2_chars_only=True, make_dictionaries=False):
    Chem.AllChem.ComputeGasteigerCharges(m)
    names = pyrogen.add_atom_names(m)
    atom_types.set_atom_types(m)
    atom_types.set_parmfrosst_atom_types(m)
    aa_types=[]

    if make_dictionaries:
       cif_file_name = "test-pyrogen-"+str(mol_idx)+'.cif'
       comp_id = "PF_" + str(mol_idx)
       pysw.mmcif_dict_from_mol(comp_id, m.GetProp('_Name'), m, cif_file_name, False, False, True)

    for idx,atom in enumerate(m.GetAtoms()):
        try:
            # in pathological cases, pf_atom_type is not set
            coot_amber_type = atom.GetProp('pf_atom_type')
            # if we were passed ref_types, compare the atom type
            name = atom.GetProp('name')
            ref_type=ref_types[idx]
            if ref_type == "Nstar":
               ref_type = 'N*'
            if ref_type == "Cstar":
               ref_type = 'C*'
            check_coot_amber_type = coot_amber_type
            if check_2_chars_only:
                check_coot_amber_type = coot_amber_type[:2]
                # C -> Ca or Cb or Cc etc is a special case
                if coot_amber_type in ["Ca", "Cax", "Cb", "Cc", "Cd", "Ce", "Cf", "Cg", "Ch", "Ci", "Cj", "C-"]:
                    check_coot_amber_type = "C"
                if coot_amber_type in ["Sa", "Sax", "Sb", "Sc", "Sd", "Se", "Sf", "Sg", "Sh", "Si", "Sj"]:
                    check_coot_amber_type = "S"
            if check_coot_amber_type != ref_type:
                ct = compare_types(atom)
                match_result = ct.compare(check_coot_amber_type, ref_type)
                conf = m.GetConformer()
                coords = conf.GetAtomPosition(idx)
                cc = "(set-rotation-centre " + \
                     str(coords.x) + ' ' + str(coords.y) + ' ' + str(coords.z) + ')'
                s = 'failed to match mol_idx {:>2} atom_idx {:>2} name {} coot_type {:>10} '
                s += "ref_type {:>2} rank {:>18} at {}"
                # Note: A match.BELOW means that we too specific in the SMILES for the
                #       coot_amber_type. To correct, make the SMILES or general
                #       or add another one that picks up this case.
                #       A match.ABOVE means that we were too general in the SMILES.
                #       This is more problematic.  We need to get rid of these by
                #       making the SMILES more specific
                print(s.format(mol_idx, idx, name, coot_amber_type, ref_type, match_result, cc))
            else:
                s = 'matched         mol_idx {} atom_idx {:>2} name {} coot_type {:>4} ref_type {}'
                print(s.format(mol_idx, idx, name, coot_amber_type, ref_type))

            p=(atom.GetProp('name'), coot_amber_type)
            aa_types.append(p)
        except TypeError as e:
            print('TypeError {}'.format(e))
            pass
        except KeyError as e:
            # pf_atom_type was not set on an atom
            print('KeyError {}'.format(e))
            pass
    return aa_types



if __name__ == '__main__':

   top_n = 100

   # write_mols(suppl) # only need do this once

   suppl = Chem.SDMolSupplier('parm/zinc-frag-100.sdf', removeHs=False)
   n_tot = 0
   ref_idx_offset={}
   for i,m in enumerate(suppl):
       n = m.GetNumAtoms()
       ref_idx_offset[i]=(n_tot, n_tot+n)
       n_tot +=n
       # print(i,n, n_tot)
   # print(ref_idx_offset)

   f = open('parm/zinc_p_f_types.txt')
   ref_types=[line.strip() for line in f.readlines()]
   f.close()

   for sdf_index in range(top_n):
      ref_types_for_mol=ref_types[ref_idx_offset[sdf_index][0]:ref_idx_offset[sdf_index][1]]
      types = get_amber_types(suppl[sdf_index], sdf_index, ref_types_for_mol, check_2_chars_only=True)
#



#!/usr/bin/python
import socket
import sys

if len(sys.argv) < 3:
    print 'Usage: %s [hostname] [portnumber]' % sys.argv[0]
    sys.exit(1)

try:
    hostname = sys.argv[1]
    port = int(sys.argv[2])
except:
    print 'Usage: %s [hostname] [portnumber]' % sys.argv[0]
    sys.exit(1)

def make_server_for_remote_control():
    global sock
    msg = 'start'
    print "BL DEBUG:: msg, host, port", msg, hostname,port
    #Setup a standard internet socket.
    #The sockopt call lets this server use the given port even if
    #it was recently used by another server
    sock = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
    sock.setsockopt(socket.SOL_SOCKET,socket.SO_REUSEADDR,1)
    sock.bind((hostname,port))
    sock.listen(1)
    print 'Waiting for a Request'

    #Handle a client request
    request, clientAddress = sock.accept()
    print 'Request received from: ', clientAddress
    data = request.recv(1024)
    print 'Received connection Msg: ', data
    while msg != 'exit':
        msg = raw_input('Enter a Message: ')
        request.send(msg + "# end")
        data = request.recv(1024)
        print 'Received Msg: ', data
    request.shutdown(2) #Stop the client from reading or writing anything.
    sock.close()
    
make_server_for_remote_control()



#    back_ground_demo.py
#    Copyright (C) 2006  Bernhard Lohkamp
#    Copyright (C) 2006  Paul Emsley, The University of York
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#;; A silly (but somewhat amusing) demo that changes the background colour
#;; over a range of colours.
#;;
#;; The scene is spun too, for extra eye-candy (so have a molecule
#;; loaded before you run this).
#;;
#;; What's the point?  It's to show that we can display the result of
#;; an arbitarily complex computation, i.e, we have a real extension
#;; language, not just a list of commands (like some other molecular
#;; graphics programs).

# n-steps is divided into 3 sets:
n_steps = 300.0

# flash the background different colours in some uninteresting way.
def background_demo():

    def rgb_colours(nlocal):

        r_step = 1 / n_steps
        red = 0.0
        green =0.0
        blue = 0.0

        if (n_local < (n_steps / 3)):
            red = 3 * r_step * n_local
            green = 0.0
        elif (n_local < ((2 * n_steps) / 3)):
            red = 1 - ((n_local - (n_steps / 3)) * 3 * r_step)
            green = ((n_local - (n_steps / 3)) * 3 * r_step)
        else:
            red = 0.0
            green = 1 - (n_local - ((n_steps / 3) * 2)) * 3 * r_step

        return red,green,blue


    n_local = 1.0
    while (n_local<=n_steps):
        r_col = rgb_colours(n_local)
        coot.set_background_colour(r_col[0],r_col[1],r_col[2])
        coot.rotate_y_scene(10,0.1) # n-frames frame-interval(degrees)
        n_local = n_local + 1.0

background_demo()



# create_server.py

import socket
import sys

hostname = sys.argv[1]
port = int(sys.argv[2])

def connection_proc(port, hostname):

    close_transmission_string = "# close\n# end\n"
    end_transmission_string = "# end\n"

    def end_transmission():
        request.send(end_transmission_string)

    def close_transmission():
        request.send(close_transmission_string)

    # main body

    print("BL DEBUG:: host, port", hostname,port)
    #Setup a standard internet socket.
    #The sockopt call lets this server use the given port even if
    #it was recently used by another server
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR,1)
    sock.bind((hostname,port))
    sock.listen(1)

    #Handle a client request
    request, clientAddress = sock.accept()
    print("################ Got a connection!")

    request.send("\n")

    print("BL DEBUG:: read ACT")
    request.send("read_pdb('monomer-ACT.pdb')\n")
    end_transmission()

    import time
    time.sleep(5)
    # more!?
    print("BL DEBUG:: read PIN")
    request.send("imol = coot.read_pdb('monomer-PIN.pdb')\n")
    request.send("set_rotation_centre(5, 5, 5)\n")
    request.send("move_molecule_to_screen_centre(imol)\n")
    end_transmission()
    # close transmission
    print("BL DEBUG:: send close")
    close_transmission()
    sock.close()

    #data = request.recv(1024)
    #print 'Received connection Msg: ', data
    #while msg != 'exit':
    #    msg = raw_input('Enter a Message: ')
    #    request.send(msg)
    #    data = request.recv(1024)
    #    print 'Received Msg: ', data
    #request.shutdown(2) #Stop the client from reading or writing anything.

connection_proc(port, hostname)



#!/usr/bin/env python
"""Extend an existing chain of nucleotides."""

# Copyright 2012 Kevin Keating
# 
# Licensed under the Educational Community License, Version 2.0 (the
# "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at
# 
# http://www.osedu.org/licenses/ECL-2.0
# 
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.

from coot import add_status_bar_text, clear_pending_picks, imol_refinement_map
from coot import user_defined_click_py as user_defined_click

from selectMapDialog import selectMapDialog
from citationPopup import createCitationPopup
from pseudoMolecule import PseudoMolecule
import rotamerize #we only need clearPendingRotamerize, but if we import that function directly into this namespace
    #we'll wind up with circular dependencies
from traceGui import TraceGui
from reportInputErrors import reportPDB2Error, reportInsCodeError, STANDARD_BASES, reportModifiedNucError


__menuItem = None #the "Add to terminal nts..." menuItem object
waitingForClicks = False #whether or not we are waiting for the user to pick an atom


def pickAtoms():
    """Prompt the user to pick an atoms to extend a stretch of nucleotides.  Alternatively, if
    we're already waiting for the user to pick an atom, then cancel the pending pick.
    
    ARGUMENTS:
        None
    RETURNS:
        None
    """
    
    #create the citation pop-up if necessary
    createCitationPopup()
    
    #make sure that we're not waiting on a pending rotamerize call
    rotamerize.clearPendingRotamerize()
    
    global waitingForClicks
    
    if waitingForClicks:
        #if we're already in the middle of an extend chain call, then cancel the pending call
        print "Rotamerize cancelled"
        add_status_bar_text("Extend chain cancelled")
        clear_pending_picks() #tell Coot to stop waiting for the user to click on atoms
        __setMenuToExtendChain()
        waitingForClicks = False
    
    else:
        #if we're not in the middle of a extend chain call, then start one
        
        #make sure that there is a refinement map set
        if imol_refinement_map() == -1:
            print "No refinement map set for RCrane extend chain"
            selectMapDialog(pickAtoms)
            return
        
        print "Click on a nucleotide to extend"
        add_status_bar_text("Pick a nucleotide [Ctrl Left-mouse rotates the view]...")
        __setMenuToCancel()
        waitingForClicks = True
        user_defined_click(1, extendChainFromAtomSpec)
    

def extendChainFromAtomSpec(atomSpec):
    """Rotamerize a stretch of nucleotides
    
    ARGUMENTS:
        atomSpec - an atom spec specifying the start of the chain extension
    RETURNS:
        False if there is a problem with the selected nucleotide
        None otherwise
    """
    
    #stop waiting for the user to cancel the rotamerize
    __setMenuToExtendChain()
    global waitingForClicks
    waitingForClicks = False
    
    #print "In extendChainFromAtomSpec"
    #print "atomSpec =", atomSpec
    
    (mol, chain, resNum, insCode, atom, alt) = atomSpec[1:]
    atom = atom.strip()
    try:
        pseudoMol = PseudoMolecule(createFromMolecule = mol, chain = chain)
    except PseudoMoleculeError, err:
        #if creating the PseudoMolecule raises an error, then it means that molecule we clicked on contains
        #anisotropic temperature records and we're using a version of Coot that's older than rev 3631
        #so clear_and_update_molecule can't handle anisotropic records
        
        print err
        #add_status_bar_text(str(err))
        
        #Since this isn't an intuitive problem (i.e. it's far less obvious for the user than "you clicked on
        #two different chains"), we create a dialog box instead of a status bar message
        createRCraneErrorDialog(str(err) + "\n\nRotamerization canceled.")
        
        return False
    
    resNumFull = str(resNum) + str(insCode)
    resIndex = pseudoMol.resIndex(resNum)
    
    #figure out what direction to extend in
    connectedToNext = pseudoMol.connectedToNextFromIndex(resIndex)
    connectedToPrev = pseudoMol.connectedToPrevFromIndex(resIndex)
    curResIsPhosOnly = pseudoMol.isOnlyPhosGroupFromIndex(resIndex)
    
    nextResIsPhosOnly = False
    if connectedToNext:
        nextResIsPhosOnly = pseudoMol.isOnlyPhosGroupFromIndex(resIndex+1)
    
    
    extendDir = None #the direction to extend the chain in
    
    if curResIsPhosOnly:
        if connectedToPrev:
            #we're building on a 3' phosphate
            extendDir = 3
        else:
            #the selected residue is an isolated phosphate group, so we can't do anything
            print "Nucleotide " + resNumFull + " is an isolated phosphate group.  Cannot extend."
            add_status_bar_text("Nucleotide " + resNumFull + " is an isolated phosphate group.  Cannot extend.")
            return False
    elif nextResIsPhosOnly:
        if connectedToPrev:
            #we're at the 3' end of the chain and there is a 3' phosphate
            extendDir = 3
            resIndex += 1 #increment the resIndex so that it points to the real last residue of the segment
            curResIsPhosOnly = True
            resNumFull = pseudoMol.resNumFull(resIndex) #update the residue name in case we need to use it for later error messages
            
        else:
            #we're dealing with a single nt containing a 3' phosphate
            #we need to decide direction based on the atom clicked
            
            extendDir = __decideDirectionFromAtom(atom, resNumFull)
            if not extendDir: return False #if we couldn't determine a direction from the atom, then cancel
    
    elif connectedToNext and connectedToPrev:
        print "Nucleotide " + resNumFull + " is not at the end of a chain"
        add_status_bar_text("Nucleotide " + resNumFull + " is not at the end of a chain")
        return False
    elif connectedToNext:
        extendDir = 5
    elif connectedToPrev:
        extendDir = 3
    else:
        #we're dealing with a single nt not containing a 3' phosphate
        #we need to decide direction based on the atom clicked
        extendDir = __decideDirectionFromAtom(atom, resNumFull)
        if not extendDir: return False #if we couldn't determine a direction from the atom, then cancel
    
    
    
    #make sure that the residue we're going to extend doesn't have an insertion code
    #   (also check the next and previous residues)
    if pseudoMol.resNum(resIndex)[1] != "":
        reportInsCodeError(resNumFull)
        return False
    if connectedToNext and pseudoMol.resNum(resIndex+1)[1] != "":
        reportInsCodeError(pseudoMol.resNumFull(resIndex+1))
        return False
    if connectedToPrev and pseudoMol.resNum(resIndex-1)[1] != "":
        reportInsCodeError(pseudoMol.resNumFull(resIndex-1))
        return False
    
    #figure out which residue we should be checking for PDB2 naming and missing glycosidic bond atoms
    if curResIsPhosOnly:
        resIndexToCheck = resIndex - 1
        resNumFullToCheck = pseudoMol.resNumFull(resIndex - 1)
    else:
        resIndexToCheck = resIndex
        resNumFullToCheck = resNumFull
    
    #make sure that this nucleotide isn't mdofied
    resType = pseudoMol.resTypeFromIndex(resIndexToCheck)
    if resType not in STANDARD_BASES:
        reportModifiedNucError(resNumFullToCheck, resType)
        return False
    
    #make sure that the molecule (or at least the end of it we're going to be building on) uses PDB3 atom naming
    if pseudoMol.checkPDB2FromIndex(resIndexToCheck):
        reportPDB2Error(resNumFullToCheck)
        return False
    
    #the current residue must have a glycosidic bond
    baseType = pseudoMol.resTypeFromIndex(resIndexToCheck)
    if baseType == "A" or baseType== "G":
        glyN = "N9"
    else:
        #we already know that the base isn't modified because we checked above
        glyN = "N1"
    
    if pseudoMol.getAtomCoordsFromIndex("C1'", resIndexToCheck) is None:
        print "Nucleotide " + resNumFullToCheck + " does not have a C1' atom.  Cannot extend chain."
        add_status_bar_text("Nucleotide " + resNumFullToCheck + " does not have a C1' atom.  Cannot extend chain.")
        return False
    if pseudoMol.getAtomCoordsFromIndex(glyN, resIndexToCheck) is None:
        print "Nucleotide " + resNumFullToCheck + " does not have an " + glyN + " atom.  Cannot extend chain."
        add_status_bar_text("Nucleotide " + resNumFullToCheck + " does not have an " + glyN + " atom.  Cannot extend chain.")
        return False
    
    #make sure that this isn't an isolated nt without any phosphates
    curPhosCoords = pseudoMol.getPhosCoordsFromIndex(resIndex) #only used for sanity checking
    if not connectedToNext and not connectedToPrev and curPhosCoords is None:
        print "Nucleotide " + resNumFull + "is an isolated nucleotides without any phosphates.  Cannot extend chain."
        add_status_bar_text("Nucleotide " + resNumFull + "is an isolated nucleotides without any phosphates.  Cannot extend chain.")
        return False
    
    #make sure that we're not building 3'->5' on a nucleotide without a 3' phosphate (because we can't determine pseudotorsions w/o a 3' phos)
    if extendDir == 5 and not connectedToNext:
        print "Cannot extend nucleotide " + resNumFull + " in the 5' direction without a 3' phosphate."
        print "  Please add a 3' phosphate by extending the chain in the 3' direction."
        add_status_bar_text("Cannot extend nucleotide " + resNumFull + " in the 5' direction without a 3' phosphate.")
        return False
    
    print "About to extend the chain in the " + str(extendDir) + "' direction"
    
    #tell the PseudoMolecule object where to insert new residues
    pseudoMol.setResInsertionPoint(resIndex)
    
    #save the molecule state so we can restore it if we cancel the extend
    pseudoMol.saveMoleculeState()
    
    TraceGui(direction = extendDir, existingMolecule = pseudoMol, resIndexToExtend = resIndex)
    


ATOMS3P = frozenset(["C3'", "O3'"])
    #atoms that count as a 3' selection
ATOMS5P = frozenset(["P", "OP1", "OP2", "OP3", "O5'", "C5'", "C4'"])
    #atoms that count as a 5' selection
def __decideDirectionFromAtom(atomName, resNum):
    """Determine the direction to extend the chain based on the atom name (for use with isolated nuclotides)
    
    ARGUMENTS:
        atomName - the atom name
        resNum   - the residue number of the current nucleotide (used in reporting errors)
    RETURNS:
        False if the direction cannot be determined from the atom name
        3 or 5 otherwise
    EFFECTS:
        if the direction cannot be determined from the atom name, an error will be reported to the user
        (and False will be returned)
    """
    
    if atomName in ATOMS3P:
        return 3
    elif atomName in ATOMS5P:
        return 5
    else:
        print "Nucleotide " + resNum + " is an isolated nucleotide.  Please select either the O5' or"
        print "O3' atom to extend in the 5' or 3' direction, respectively."
        add_status_bar_text("Must select either O5' or O3' of isolated nucleotides.")
        return False


def storeMenuItem(menuItem, resetLabel = False):
    """ARGUMENTS:
        menuItem   - the gtk MenuItem to store
    OPTIONAL ARGUMENTS:
        resetLabel    - whether we should reset the menu label to "Extend chain..."
                        (and also clear all pending atom picks in Coot)
                        This is intended for use when we're reloading RCrane
                        Defaults to False
    RETURNS:
        None
    EFFECTS:
        stores menuItem in the module-level variable __menuItem
    """
    
    global __menuItem
    __menuItem = menuItem
    
    #if we're being called during an RCrane reload, then reset everything back to the appropriate starting position
    global waitingForClicks
    if resetLabel:
        waitingForClicks = False #this is redundant with module initialization, but can't hurt to do
                                 #in case this function is called in an unexpected way
        __setMenuToExtendChain()
        clear_pending_picks()


def __setMenuToExtendChain():
    """Set the menu entry labels to "Extend chain..."
    
    ARGUMENTS:
        None
    RETURNS:
        None
    """
    
    if __menuItem is not None:
        __menuItem.get_child().set_text("Extend chain...")


def __setMenuToCancel():
    """Set the menu entry label to "Cancel extend chain..."
    
    ARGUMENTS:
        None
    RETURNS:
        None
    """
    
    if __menuItem is not None:
        __menuItem.get_child().set_text("Cancel extend chain...")


def clearPendingExtendChain():
    """Cancel any pending extend chain call (intended to be called by rotamerize)
    
    ARGUMENTS:
        None
    RETURNS:
        True if there was a pending extend chain call
        False otherwise
    """
    
    global waitingForClicks
    
    if waitingForClicks:
        #if we're already in the middle of a extend chain call, then cancel the pending extend chain
        print "Extend chain cancelled"
        clear_pending_picks() #tell Coot to stop waiting for the user to click on atoms
        __setMenuToExtendChain()
        waitingForClicks = False
        return True
    else:
        return False



#    03_ligand.oy
#    Copyright (C) 2008  Bernhard Lohkamp, The University of York
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import coot
import coot_utils
import begin
import unittest
import os

class TestLigandFunctions(unittest.TestCase):

    def test01_0(self):
        """Get monomer test"""

        imol = coot.get_monomer("3GP")
        if (coot_utils.valid_model_molecule_qm(imol)):
            coot.delete_residue_hydrogens(imol, "A", 1, "", "")
        self.assertFalse(not coot_utils.valid_model_molecule_qm(imol), "not valid molecule for 3GP")


    def test03_0(self):
        """Delete all-molecule Hydrogens"""
        imol = begin.unittest_pdb("monomer-3GP.pdb")
        self.assertTrue(coot_utils.valid_model_molecule_qm(imol))
        print("here 1")
        n = coot.delete_hydrogens(imol)
        self.assertTrue(n > 0)


    def test04_0(self):
        """Non-Autoloads"""

        def get_ccp4_version():
            s = coot_utils.shell_command_to_string("cad -i").split("\n")
            if s:
                for line in s:
                    if "CCP4 software suite: patch level" in line:
                        sl = line.split()
                        return sl[-1]
            return False

        def old_ccp4_restraints_qm():
            global have_ccp4_qm
            if (not have_ccp4_qm):
                return False
            else:
                v = get_ccp4_version()
                # will always be string
                return v < "6.2"

        r_1 = coot.monomer_restraints("LIG")
        o = old_ccp4_restraints_qm()
        begin.unittest_pdb("test-LIG.pdb")
        r_2 = coot.monomer_restraints("LIG")
        coot.remove_non_auto_load_residue_name("LIG")
        begin.unittest_pdb("test-LIG.pdb")
        r_3 = coot.monomer_restraints("LIG")
        coot.delete_restraints("LIG")
        coot.add_non_auto_load_residue_name("LIG")
        r_4 = coot.monomer_restraints("LIG")
        # r_1, r_2, r_4 should be False, r_3 should be filled
        #
        # just to be clear here: either this is modern
        # restraints and we should get a list in r-3... or
        # this is old restraints. Either of those should
        # result in a PASS.
        self.assertTrue(all([r_1 == False,
                             r_2 == False,
                             r_4 == False,
                             old_ccp4_restraints_qm() if \
                             old_ccp4_restraints_qm() else \
                             isinstance(r_3, dict)]))


    def test05_0(self):
        """Merge molecules of a ligand with a spec"""

        imol = begin.unittest_pdb("tutorial-modern.pdb")
        imol_lig = coot.get_monomer("3GP")

        spec = ["L", 1, ""]

        coot.set_merge_molecules_ligand_spec(spec)
        coot.merge_molecules([imol_lig], imol)

        # now check that L1 exists in imol

        # residue_spec_to_residue_name expects a 4-ele spec. Hmm
        spec[0:0] = [True]
        rn = coot_utils.residue_spec_to_residue_name(imol, spec)
        self.assertTrue(isinstance(rn, str))
        self.assertTrue(rn == "3GP")


    def test05_1(self):
        """Move and Refine Ligand test"""

        imol = begin.unittest_pdb("tutorial-modern.pdb")
        self.assertTrue(coot_utils.valid_model_molecule_qm(imol))
        new_rc = [55.3, 9.1, 20.6]
        # set the view
        view_number = coot.add_view([54.5698, 8.7148, 20.5308],
                               [0.046229, -0.157139, -0.805581, 0.569395],
                               19.8858,
                               "ligand-view")

        coot.go_to_view_number(view_number, 1)

        # update the map
        coot.set_rotation_centre(*new_rc)
        coot.move_molecule_here(imol)
        backup_mode = coot.backup_state(imol)
        alt_conf = ""
        replacement_state = coot.refinement_immediate_replacement_state()

        coot.turn_off_backup(imol)
        coot.set_refinement_immediate_replacement(1)
        coot.refine_zone(imol, "A", 1, 1, alt_conf)
        coot.accept_regularizement()
        coot.rotate_y_scene(600, 0.1)
        if replacement_state == 0:
            coot.set_refinement_immediate_replacement(0)
        if backup_mode == 1:
            coot.turn_on_backup(imol)
        # ok. no fail.


    def test06_0(self):
        """Many Molecules - Ligand Fitting"""

        npo_pdb     = os.path.join(begin.unittest_data_dir, "monomer-NPO.pdb")
        pdb43ca_pdb = os.path.join(begin.unittest_data_dir, "pdb43ca-sans-NPO-refmaced.pdb")
        pdb43ca_mtz = os.path.join(begin.unittest_data_dir, "pdb43ca-sans-NPO-refmaced.mtz")
        imol_npo = coot.handle_read_draw_molecule_with_recentre(npo_pdb, 0)

        self.assertTrue(coot_utils.valid_model_molecule_qm(imol_npo), "no valid molecule")

        for i in range(0, 5):
            imol_copy = coot.copy_molecule(imol_npo)
            coot.set_mol_displayed(imol_copy, 0)

        imol_protein = coot.read_pdb(pdb43ca_pdb)
        imol_map = coot.auto_read_make_and_draw_maps(pdb43ca_mtz)
        imol_map_1 = imol_map[0]
        imol_map_2 = imol_map[1]

        coot.add_ligand_clear_ligands()
        coot.set_ligand_search_protein_molecule(imol_protein)
        coot.set_ligand_search_map_molecule(imol_map_1)
        coot.add_ligand_search_ligand_molecule(imol_npo)

        solutions = coot.execute_ligand_search()   # crash?
        print("   Fitting NPO gave these results: ", solutions)
        coot.set_map_displayed(imol_map_1, 0)
        coot.set_map_displayed(imol_map_2, 0)
        coot.set_mol_displayed(imol_protein, 0)
        coot.set_mol_displayed(imol_npo, 0)
        # checked for non crash


    def test07_0(self):
        """flip residue (around eigen vectors)"""

        # new version
        imol_orig = begin.unittest_pdb("monomer-3GP.pdb")
        imol_copy = coot.copy_molecule(imol_orig)

        self.assertFalse(not coot_utils.valid_model_molecule_qm(imol_orig),
                    "not valid molecule for monomer-3GP.pdb")

        # we need this, otherwise active-atom is (accidentally) the wrong
        # molecule
        coot.set_go_to_atom_molecule(imol_copy)
        coot.set_go_to_atom_chain_residue_atom_name("A", 1, " C8 ")

        active_atom = coot.active_residue()
        self.assertTrue(active_atom, "No active atom")
        imol      = active_atom[0]
        chain_id  = active_atom[1]
        res_no    = active_atom[2]
        ins_code  = active_atom[3]
        atom_name = active_atom[4]
        alt_conf  = active_atom[5]
        self.assertFalse(imol == imol_orig, "oops - didn't pick the copy for active res")
        coot.flip_ligand(imol, chain_id, res_no)
        atom_orig_1 = coot.get_atom(imol_orig, "A", 1, "", " C8 ")
        atom_move_1 = coot.get_atom(imol     , "A", 1, "", " C8 ")

        self.assertTrue(isinstance(atom_orig_1, list), "atom_orig_1 not found")

        self.assertTrue(isinstance(atom_move_1, list), "atom_move_1 not found")

        d = coot_utils.bond_length(atom_orig_1[2], atom_move_1[2])
        print("distance: ", d)
        self.assertTrue(d > 2.1, "fail to move test atom d1")
        coot.flip_ligand(imol, chain_id, res_no)
        coot.flip_ligand(imol, chain_id, res_no)
        coot.flip_ligand(imol, chain_id, res_no)
        # having flipped it round the axes 4
        # times, we should be back where we
        # started.
        atom_orig_1 = coot.get_atom(imol_orig, "A", 1, "", " C8 ")
        atom_move_1 = coot.get_atom(imol     , "A", 1, "", " C8 ")
        d2 = coot_utils.bond_length(atom_orig_1[2], atom_move_1[2])
        print("distance d2: ", d2)
        self.assertTrue(d2 < 0.001, "fail to move atom back to start d2")


    def test08_0(self):
        """Test dipole"""

        if self.skip_test(True, "Skipping dipole test. disabled for now!"):
            return

        imol = begin.unittest_pdb("dipole-residues.pdb")

        self.assertTrue(coot_utils.valid_model_molecule_qm(imol), "dipole-residues.pdb not found")

        residue_specs = [["A", 1, ""],
                         ["A", 2, ""],
                         ["A", 3, ""]]
        dipole = coot.add_dipole_for_residues(imol, residue_specs)

        self.assertFalse(not dipole, "bad dipole %s" %dipole)

        d = dipole[0]
        dip = dipole[1]

        dip_x = dip[0]
        dip_y = dip[1]
        dip_z = dip[2]

        print("info:: dipole components", dip)

        self.assertAlmostEqual(dip_y, 0.0, 2, "bad dipole y component %s" %dip_y)
        self.assertAlmostEqual(dip_z, 0.0, 2, "bad dipole z component %s" %dip_z)

        self.assertTrue(dip_x < 0 and dip_x > -20)


    def test09_0(self):
        """Reading new dictionary restraints replaces"""

        def get_torsions(r):
            return r["_chem_comp_tor"]

        coot.read_cif_dictionary(os.path.join(begin.unittest_data_dir, "libcheck_3GP.cif"))
        coot.read_cif_dictionary(os.path.join(begin.unittest_data_dir, "libcheck_3GP.cif"))
        coot.read_cif_dictionary(os.path.join(begin.unittest_data_dir, "libcheck_3GP.cif"))

        r = coot.monomer_restraints("3GP")
        self.assertTrue(r, "Failed to get restraints from monomer 3GP")
        t = coot.get_torsions(r)

        #
        self.assertTrue(len(t) < 26, "torsions: %s %s" %(len(t), t))
        # 22 in new dictionary, it seems


    def test10_0(self):
        """Pyrogen Runs OK?"""

        if self.skip_test(not coot.enhanced_ligand_coot_p(),
                          "No ligand enhanced version, skipping Pyrogen test"):
            return

        # bad things may well happen if we run the wrong version of pyrogen.
        # so force pyrogen to be the one that is installed alongside this version of coot
        # that we are running. We do that by looking and manipulating sys.argv[0]
        import os, sys

        coot_dir = os.path.abspath(os.path.dirname(sys.argv[0]))
        prefix_dir = os.path.normpath(os.path.join(coot_dir, ".."))
        pyrogen_exe = "pyrogen"
        pyrogen_bin = pyrogen_exe # test installed pyrogen
        if coot.is_windows():
            pyrogen_exe = "pyrogen.bat"
            pyrogen_bin = os.path.normpath(os.path.join(prefix_dir, "bin", pyrogen_exe))

        smiles = "C1CNC1"
        tlc_text = "XXX"
        log_file_name = "pyrogen.log"

        # do we have pass now?
        if not coot.enhanced_ligand_coot_p():
            # dont test pyrogen
            # not needed any more. Tested above.
            return
        else:

            arg_list = ["--no-mogul", "--residue-type", tlc_text, smiles] # --no-mogul is the default now
            popen_status = coot_utils.popen_command(pyrogen_bin, arg_list, [], log_file_name, True)
            # self.assertTrue(popen_status == 0)
            self.assertEqual(popen_status, 0,
                             "WARNING:: pyrogen exited with status %i\n" %popen_status)
            pdb_file_name = tlc_text + "-pyrogen.pdb"
            cif_file_name = tlc_text + "-pyrogen.cif"
            imol = coot.handle_read_draw_molecule_with_recentre(pdb_file_name, 0)
            print("INFO:: pyrogen will try to read pdb file %s" %pdb_file_name)
            # add test for chirality in the dictionary here
            self.assertTrue(coot_utils.valid_model_molecule_qm(imol))

    def test11_0(self):
        """pyrogen dictionary does not make double-quoted atom names"""

        # untested - no mogul

        # make sure that you are running the correct pyrogen

        if self.skip_test(not coot.enhanced_ligand_coot_p(),
                          "No ligand enhanced version, skipping Pyrogen test"):
            return

        import os
        if os.path.isfile("UVP-pyrogen.cif"):
            os.remove("UVP-pyrogen.cif")

        popen_status = coot_utils.popen_command("pyrogen",
#                                     ["-nM", "-r", "UVP",
                                     ["-n", "-r", "UVP",
                                      "CO[C@@H]1[C@H](O)[C@H](O[C@H]1[n+]1ccc(O)nc1O)\\C=C\\P(O)(O)=O"],
                                     [], "pyrogen.log", False)
        self.assertEqual(popen_status, 0,
                         "Fail to correctly run pyrogen\n")
        coot.read_cif_dictionary("UVP-pyrogen.cif")
        imol = coot.get_monomer("UVP")
        self.assertTrue(coot_utils.valid_model_molecule_qm(imol),
                        "Fail to load molecule from pyrogen dictionary\n")
        atom_info = coot.residue_info(imol, "A", 1, "")

    # FLEV will not make a PNG if it is not compiled with
    # C++-11 - and that is OK for 0.8.9.x.
    #
    # def test12_0(self):
    #     """FLEV makes a PNG"""

    #     import os

    #     if self.skip_test(not enhanced_ligand_coot_p(),
    #                       "No ligand enhanced version, skipping FLEV test"):
    #         return

    #     fn = "test-flev-greg-testcase.png"

    #     if os.path.exists(fn):
    #         os.remove(fn)

    #     imol = unittest_data_dir("tutorial-modern.pdb")
    #     imol_ligand = get_monomer("3GP")

    #     set_rotation_centre(54, 10, 20)
    #     move_molecule_to_screen_centre(imol_ligand)
    #     set_merge_molecules_ligand_spec(["L", 1, ""])
    #     merge_molecules([imol_ligand], imol)
    #     fle_view_with_rdkit_to_png(imol, "L", 1, "", 4.8, fn)
    #     self.failUnless(os.path.exists(fn))



#!/usr/bin/env python
"""Functions for the About dialog box."""

# Copyright 2010, 2011, 2012 Kevin Keating
# 
# Licensed under the Educational Community License, Version 2.0 (the
# "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at
# 
# http://www.osedu.org/licenses/ECL-2.0
# 
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.

import gtk
import os.path

from guiUtils import rcranePath

def createAboutDialog():
    """Creates and displays an About dialog.
    
    ARGUMENTS:
        None
    RETURNS:
        None
    EFFECTS:
        creates and displays the About dialog
    """
    
    aboutDialog = gtk.AboutDialog()
    aboutDialog.set_name("RCrane")
    aboutDialog.set_version("1.1")
    aboutDialog.set_copyright("Copyright 2010-2012 by Kevin Keating")
    aboutDialog.set_website("http://www.pylelab.org/software/index.html")
    aboutDialog.set_license(license)
    aboutDialog.set_logo(gtk.gdk.pixbuf_new_from_file(os.path.join(rcranePath, "logo.svg")))
    aboutDialog.set_icon_from_file(os.path.join(rcranePath, "icon.svg"))
    
    try:
        aboutDialog.set_wrap_license(True)
    except AttributeError:
        #if we're using PyGTK 2.6 or 2.7, we have AboutDialogs but no set_wrap_license function
        #(if we're using a PyGTK older than 2.6, we have no AboutDialog, so we've got bigger problems)
        aboutDialog.set_property("wrap-license", True)
    
    #if we have a version.py, then include the SVN revision number in the about box
    try:
        from revisionNum import revNum
        revAndComments = "Revision " + revNum + comments
    except ImportError:
        #if there's no version.py, then don't include the SVN revision number
        revAndComments = comments
    aboutDialog.set_comments(revAndComments)
    
    #add a reference button
    #this is the same way that Coot adds a reference button to it's about dialog
    refButton = gtk.Button("References")
    refButton.connect("clicked", __createReferenceDialog, aboutDialog)
    aboutDialog.action_area.pack_start(refButton, False, True, 0)
    aboutDialog.action_area.set_child_secondary(refButton, True)
    aboutDialog.action_area.reorder_child(refButton, 1)
    refButton.show()
    
    aboutDialog.run()
    aboutDialog.destroy() #this destroys the aboutDialog variable, but not the dialog box itself
        #if you don't call this, then the Close button won't work
    

def __createReferenceDialog(widget, aboutDialog):
    """Creates and displayes the References dialog (launched from the About dialog).
    
    ARUGMENTS:
        widget      - the button used to invoke the function
        aboutDialog - the window object for the About dialog
    RETURNS:
        None
    EFFECTS:
        creates and displays the References dialog
    """
    
    refDialog = gtk.Dialog("RCrane References", aboutDialog, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR)
    refDialog.resize(500, 300)
        #Coot uses 500, 400 for its reference dialog
    refDialog.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
    
    textView = gtk.TextView()
    textView.set_editable(False)
    textView.set_cursor_visible(False)
    textView.set_wrap_mode(gtk.WRAP_WORD)
    textView.set_left_margin(10)
    textView.get_buffer().set_text(references)
    textView.show()
    
    #mimic the textbox style of the license window
    scrolledWindow = gtk.ScrolledWindow()
    scrolledWindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
    scrolledWindow.set_shadow_type(gtk.SHADOW_IN)
    scrolledWindow.set_border_width(10)
    scrolledWindow.add(textView)
    
    refDialog.vbox.pack_start(scrolledWindow, True, True, 0)
    scrolledWindow.show()
    
    refDialog.run()
    refDialog.destroy()
    
    

comments = \
"""
RCrane allows for accurate and semi-automated building of RNA structure into electron density maps."""


references =\
"""All publications resulting from the use of RCrane should acknowledge:

Keating KS and Pyle AM.  RCrane: Semi-automated RNA model building.  Acta Cryst D68 (2012) 958-995.


Additional information about the RCrane methodology may be found in:

Keating KS and Pyle AM. Semiautomated model building for RNA crystallography using a directed rotameric approach. Proc Natl Acad USA, 107 (2010) 8177-8182.


Additional information on the RNA backbone conformers may be found in:

Richardson JS, Schneider B, Murray LW, Kapral GJ, Immormino RM, Headd JJ, Richardson DC, Ham D, Hershkovotz E, Williams LD, Keating KS, Pyle AM, Micallef D, Westbrook J, Berman HM. RNA backbone: Consensus all-angle conformers and modular string nomenclature (an RNA Ontology Consortium contribution).  RNA, 14 (2008) 465-481."""


license = \
"""                 Educational Community License
                       Version 2.0, April 2007

The Educational Community License version 2.0 ("ECL") consists of the Apache 2.0 license, modified to change the scope of the patent grant in section 3 to be specific to the needs of the education communities using this license. The original Apache 2.0 license can be found at: http://www.apache.org/licenses/LICENSE-2.0

TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION
1. Definitions.

"License" shall mean the terms and conditions for use, reproduction, and distribution as defined by Sections 1 through 9 of this document.

"Licensor" shall mean the copyright owner or entity authorized by the copyright owner that is granting the License.

"Legal Entity" shall mean the union of the acting entity and all other entities that control, are controlled by, or are under common control with that entity. For the purposes of this definition, "control" means (i) the power, direct or indirect, to cause the direction or management of such entity, whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the outstanding shares, or (iii) beneficial ownership of such entity.

"You" (or "Your") shall mean an individual or Legal Entity exercising permissions granted by this License.

"Source" form shall mean the preferred form for making modifications, including but not limited to software source code, documentation source, and configuration files.

"Object" form shall mean any form resulting from mechanical transformation or translation of a Source form, including but not limited to compiled object code, generated documentation, and conversions to other media types.

"Work" shall mean the work of authorship, whether in Source or Object form, made available under the License, as indicated by a copyright notice that is included in or attached to the work (an example is provided in the Appendix below).

"Derivative Works" shall mean any work, whether in Source or Object form, that is based on (or derived from) the Work and for which the editorial revisions, annotations, elaborations, or other modifications represent, as a whole, an original work of authorship. For the purposes of this License, Derivative Works shall not include works that remain separable from, or merely link (or bind by name) to the interfaces of, the Work and Derivative Works thereof.

"Contribution" shall mean any work of authorship, including the original version of the Work and any modifications or additions to that Work or Derivative Works thereof, that is intentionally submitted to Licensor for inclusion in the Work by the copyright owner or by an individual or Legal Entity authorized to submit on behalf of the copyright owner. For the purposes of this definition, "submitted" means any form of electronic, verbal, or written communication sent to the Licensor or its representatives, including but not limited to communication on electronic mailing lists, source code control systems, and issue tracking systems that are managed by, or on behalf of, the Licensor for the purpose of discussing and improving the Work, but excluding communication that is conspicuously marked or otherwise designated in writing by the copyright owner as "Not a Contribution."

"Contributor" shall mean Licensor and any individual or Legal Entity on behalf of whom a Contribution has been received by Licensor and subsequently incorporated within the Work.

2. Grant of Copyright License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright license to reproduce, prepare Derivative Works of, publicly display, publicly perform, sublicense, and distribute the Work and such Derivative Works in Source or Object form.

3. Grant of Patent License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable (except as stated in this section) patent license to make, have made, use, offer to sell, sell, import, and otherwise transfer the Work, where such license applies only to those patent claims licensable by such Contributor that are necessarily infringed by their Contribution(s) alone or by combination of their Contribution(s) with the Work to which such Contribution(s) was submitted. If You institute patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Work or a Contribution incorporated within the Work constitutes direct or contributory patent infringement, then any patent licenses granted to You under this License for that Work shall terminate as of the date such litigation is filed. Any patent license granted hereby with respect to contributions by an individual employed by an institution or organization is limited to patent claims where the individual that is the author of the Work is also the inventor of the patent claims licensed, and where the organization or institution has the right to grant such license under applicable grant and research funding agreements. No other express or implied licenses are granted.

4. Redistribution.

You may reproduce and distribute copies of the Work or Derivative Works thereof in any medium, with or without modifications, and in Source or Object form, provided that You meet the following conditions:

You must give any other recipients of the Work or Derivative Works a copy of this License; and

You must cause any modified files to carry prominent notices stating that You changed the files; and

You must retain, in the Source form of any Derivative Works that You distribute, all copyright, patent, trademark, and attribution notices from the Source form of the Work, excluding those notices that do not pertain to any part of the Derivative Works; and

If the Work includes a "NOTICE" text file as part of its distribution, then any Derivative Works that You distribute must include a readable copy of the attribution notices contained within such NOTICE file, excluding those notices that do not pertain to any part of the Derivative Works, in at least one of the following places: within a NOTICE text file distributed as part of the Derivative Works; within the Source form or documentation, if provided along with the Derivative Works; or, within a display generated by the Derivative Works, if and wherever such third-party notices normally appear. The contents of the NOTICE file are for informational purposes only and do not modify the License. You may add Your own attribution notices within Derivative Works that You distribute, alongside or as an addendum to the NOTICE text from the Work, provided that such additional attribution notices cannot be construed as modifying the License.

You may add Your own copyright statement to Your modifications and may provide additional or different license terms and conditions for use, reproduction, or distribution of Your modifications, or for any such Derivative Works as a whole, provided Your use, reproduction, and distribution of the Work otherwise complies with the conditions stated in this License.

5. Submission of Contributions. Unless You explicitly state otherwise, any Contribution intentionally submitted for inclusion in the Work by You to the Licensor shall be under the terms and conditions of this License, without any additional terms or conditions. Notwithstanding the above, nothing herein shall supersede or modify the terms of any separate license agreement you may have executed with Licensor regarding such Contributions.

6. Trademarks. This License does not grant permission to use the trade names, trademarks, service marks, or product names of the Licensor, except as required for reasonable and customary use in describing the origin of the Work and reproducing the content of the NOTICE file.

7. Disclaimer of Warranty. Unless required by applicable law or agreed to in writing, Licensor provides the Work (and each Contributor provides its Contributions) on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining the appropriateness of using or redistributing the Work and assume any risks associated with Your exercise of permissions under this License.

8. Limitation of Liability. In no event and under no legal theory, whether in tort (including negligence), contract, or otherwise, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall any Contributor be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this License or out of the use or inability to use the Work (including but not limited to damages for loss of goodwill, work stoppage, computer failure or malfunction, or any and all other commercial damages or losses), even if such Contributor has been advised of the possibility of such damages.

9. Accepting Warranty or Additional Liability. While redistributing the Work or Derivative Works thereof, You may choose to offer, and charge a fee for, acceptance of support, warranty, indemnity, or other liability obligations and/or rights consistent with this License. However, in accepting such obligations, You may act only on Your own behalf and on Your sole responsibility, not on behalf of any other Contributor, and only if You agree to indemnify, defend, and hold each Contributor harmless for any liability incurred by, or claims asserted against, such Contributor by reason of your accepting any such warranty or additional liability.

END OF TERMS AND CONDITIONS"""




import os
import coot
import coot_utils
import coot_gui  # does this load probe_command?

global probe_command
probe_command = "molprobity.probe"

# this function is not at startup?

def deactivate_molecules_except(imol):
    for i in coot_utils.model_molecule_list():
        if (i != imol):
            coot.set_mol_active(i, 0)


# This will hydrogenate the active residue, not imol
# returns the probe clash score or False if no probe
#
def contact_score_ligand(imol, res_spec):

    if not coot_utils.command_in_path_qm(probe_command):
        print("WARNING:: no probe command find, so no contact score")
        return False

    deactivate_molecules_except(imol)

    chain_id = coot_utils.res_spec_to_chain_id(res_spec)
    res_no = coot_utils.res_spec_to_res_no(res_spec)
    ins_code = coot_utils.res_spec_to_ins_code(res_spec)
    ss = "//" + chain_id + "/" + str(res_no)
    imol_selection = coot.new_molecule_by_atom_selection(imol, ss)
    coot_molprobity_dir = coot_utils.get_directory("coot-molprobity")
    ligand_selection_pdb = os.path.join(coot_molprobity_dir, "tmp-selected-ligand-for-probe-" + str(imol) + ".pdb")
    protein_selection_pdb = os.path.join(coot_molprobity_dir, "tmp-protein-for-probe-" + str(imol) + ".pdb")
    dots_file_name = os.path.join(coot_molprobity_dir, "probe-" + chain_id + "-" + str(res_no) + ".dots")
    coot.set_mol_active(imol_selection, 0)
    coot.set_mol_displayed(imol_selection, 0)
    coot.set_go_to_atom_molecule(imol)
    rc = coot.residue_centre_py(imol, chain_id, res_no, ins_code)
    coot.set_rotation_centre(*rc)
    coot.hydrogenate_region(6)

    coot.write_pdb_file(imol_selection, ligand_selection_pdb)
    coot.write_pdb_file(imol, protein_selection_pdb)

    args = ["-q", "-unformated", "-once", # -once or -both
                                  # "-outside", # crashes generic_objects.probe
                                  # first pattern
                                  "CHAIN" + chain_id + " " + str(res_no),
                                  # second pattern
                                  "not " + str(res_no),
                                  # consider og33 (occ > 0.33)
                                  "-density50",
                                  ligand_selection_pdb, protein_selection_pdb]
    coot_utils.popen_command(probe_command, args, [], dots_file_name, False)

    # coot_utils.debugging!?
    coot.handle_read_draw_probe_dots_unformatted(dots_file_name, imol, 0)

    cs = coot.probe_clash_score_py(dots_file_name)
    coot.graphics_draw()
    return cs


def contact_score_ligand_func():
    with coot_utils.UsingActiveAtom() as [aa_imol, aa_chain_id, aa_res_no,
                               aa_ins_code, aa_atom_name, aa_alt_conf]:
        contact_score_ligand(aa_imol, [aa_chain_id, aa_res_no, aa_ins_code])

def coot_contact_dots_ligand_func():
    with coot_utils.UsingActiveAtom() as [aa_imol, aa_chain_id, aa_res_no,
                               aa_ins_code, aa_atom_name, aa_alt_conf]:
        coot.coot_contact_dots_for_ligand_py(aa_imol, [aa_chain_id, aa_res_no, aa_ins_code])

# not ready for public yet
def coot_all_atom_contact_dots_func():

    with coot_utils.UsingActiveAtom() as [aa_imol, aa_chain_id, aa_res_no,
                               aa_ins_code, aa_atom_name, aa_alt_conf]:

        coot_probe_object_names = ['wide-contact', 'close-contact', 'small-overlap',
                                   'big-overlap', 'H-bond', 'clashes']
        n = coot.number_of_generic_objects()
        for i in range(n):
           if generic_object_name(i) in coot_probe_object_names:
              coot.close_generic_object(i)
           else:
               print(generic_object_name(i), "is not in", coot_probe_object_names)
        coot.coot_all_atom_contact_dots(aa_imol)




import datetime

def proc():
    fn = "../rel-todo"
    f = open(fn)
    lines = f.readlines()
    add_to_other_items = True
    add_to_done_other_items = False
    n_other_items = 0
    n_done_other_items = 0
    for line in lines:
        if "--- Items ---" in line:
            add_to_other_items = True
            add_to_done_other_items = False
        if "--- Done Items ---" in line:
            add_to_other_items = False
            add_to_done_other_items = True
        arr = line.split()
        try:
            idx = arr.index(":::")
            if idx < len(arr) - 1:
                estimated_time_for_task_string = arr[idx+1]
                etft = float(estimated_time_for_task_string)
                if add_to_other_items:
                    n_other_items += etft
                if add_to_done_other_items:
                    n_done_other_items += etft
        except ValueError as e:
            pass

    return (n_other_items, n_done_other_items)


(n_other_items, n_done_other_items) = proc()

start = datetime.datetime(2025, 3, 20)
now   = datetime.datetime.now()

delta_s = int(now.strftime("%s")) - int(start.strftime("%s"))
delta_d = delta_s/(60*60*24)
# print(delta_d, n_above, n_below, n_total)
# date/time n-items-not-yet done n-items-done
print("{:.3f} {:.3f} {:.3f}".format(delta_d, n_other_items, n_done_other_items))



#!/usr/bin/env python
"""Predict rotamers from (theta, eta) values"""

# Copyright 2010 Kevin Keating
# 
# Licensed under the Educational Community License, Version 2.0 (the
# "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at
# 
# http://www.osedu.org/licenses/ECL-2.0
# 
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.

from math import radians, sin, cos, pi, sqrt, exp

DELIM = "," #the delimeter used in the input file (i.e. csv)

#constants used in the cluster data list
STARTPUCKER = 0
ENDPUCKER   = 1
CENTER      = 2
VARIANCE    = 3
ROTATION    = 4

class ThetaEtaPredictor:
    """Predict rotamers from (theta, eta) values
    
    NOTE:
        This prediction is based in-part on the soft K-means clustering algorithm from:
        MacKay, D.J.C. 2002. Maximum Likelihood and Clustering. In Information Theory, Inference & Learning
            Algorithms, pp. 300-306,  Cambridge University Press, New York, NY.
    """
    
    def __init__(self, fileName):
        """Create and initialize a ThetaEtaPredictor object
        
        ARGUMENTS:
            inputFile - a csv file containing training data for theta-eta Gaussian clusters
        RETURNS:
            a ThetaEtaPredictor object initialized with the cluster data from inputFile
        """
        
        input = open(fileName, 'r')
        input.readline() #ignore the header
        
        clusters = {}
        
        for curline in input:
            data = curline.split(DELIM)
            
            rotName     = data[0]
            startPucker = data[1]
            endPucker   = data[2]
            center      = map(float, data[3:5])
            variance    = map(float, data[5:7])
            rotation    = float(data[7])
           
            clusters[rotName] = [startPucker, endPucker, center, variance, rotation]
        
        input.close()
        
        self.__clusters = clusters
    
    
    def calcProb(self, theta, eta, rot = None):
        """Calculate the probability of each rotamer for a given (theta, eta) value
        
        ARGUMENTS:
            theta  - the theta value of the point
            eta    - the eta value of the point
        OPTIONAL ARGUMENTS
            rot    - a specific rotamer
        RETURNS:
            if rot is not given, a dictionary of {rotamer => probability}
            if rot is given, the probability that the rotamer is rot
        """
        
        probs = {}
        
        #calculate the raw (unscaled) probability for each rotamer
        for currot in self.__clusters.keys():
            probs[currot] = self.__calcRawProb(theta, eta, currot)
        
        #scale the raw probabilities so they add up to 1
        totalProb = sum(probs.values())
        probs = dict([(currot, curprob/totalProb) for (currot, curprob) in probs.iteritems()])
        
        #return either the desired probability or the dictionary containing all probabilities
        if rot is None:
            return probs
        else:
            return probs[rot]
    
    
    def __calcRawProb(self, theta, eta, rot):
        """Calculate the responsibility of a given rotamer cluster for a given point based only on (theta, eta) values
        
        ARGUMENTS:
            theta  - the theta value of the point
            eta    - the eta value of the point
            rot    - the rotamer
        RETURNS:
            the unscaled responsibility of rot for (theta, eta)
        """
        
        center   = self.__clusters[rot][CENTER]
        variance = self.__clusters[rot][VARIANCE]
        angle    = self.__clusters[rot][ROTATION]
        point = [theta, eta]
        
        #first, translate the point
        x = subtractCoords(point[0], center[0])
        y = subtractCoords(point[1], center[1])
        
        #rotate the point by the opposite of the angle
        angle = radians(angle)
        point[0] = x*cos(angle) - y*sin(angle)
        point[1] = x*sin(angle) + y*cos(angle)
        
        #the following caculation is based on the soft K-means clustering algorithm from
        #MacKay, D.J.C. 2002. Maximum Likelihood and Clustering. In Information Theory, Inference & Learning
        #    Algorithms, pp. 300-306,  Cambridge University Press, New York, NY.
        
        product = 1
        sum = 0
        for curaxis in (0, 1):
            product *= sqrt(2*pi * variance[curaxis])
            sum     += (point[curaxis]**2 / (2*variance[curaxis]))
        
        return 1.0/product * exp(-sum)


def subtractCoords(num1, num2):
    """Calculate the minimum distance between two measures, keeping the appropriate sign
    
    ARGUMENTS:
        num1 - the first measurement
        num2 - the second measurement
    RETURNS:
        the distance between these two angles of no more than 180 degrees
    """
    
    return (num1 - num2 + 180) % 360 - 180




ligand_check_refmac_mtz = False
ligand_check_refmac_fobs_col = False
ligand_check_refmac_sigfobs_col = False
ligand_check_refmac_r_free_col = False

def ligand_check_refmac_columns(f_list, sigf_list, rfree_list):

    # dummy function, not needed anywhere (as is)
    # need the above as globals...

    # happy path

    # Using the first sigf (there should only be one typically)
    # get the F label (Fx) from x/y/SIGFPx
    #
    pass

def jiggle_fit_active_residue():
    with coot_utils.UsingActiveAtom() as [aa_imol, aa_chain_id, aa_res_no,
                               aa_ins_code, aa_atom_name, aa_alt_conf]:
        coot.fit_to_map_by_random_jiggle(aa_imol, aa_chain_id, aa_res_no,
                                    aa_ins_code, 100, 1.0)

if (use_gui_qm != 2):
    menu = coot_gui.coot_menubar_menu("Ligand")

    coot_gui.add_simple_coot_menu_menuitem(
        menu,
        "Find Ligands...",
        lambda func: coot.do_find_ligands_dialog())

    coot_gui.add_simple_coot_menu_menuitem(
        menu,
        "Jiggle-Fit Ligand",
        lambda func: jiggle_fit_active_residue())

    coot_gui.add_simple_coot_menu_menuitem(
        menu,
        "Hydrogenate region",
        lambda func: coot.hydrogenate_region(6))

    coot_gui.add_simple_coot_menu_menuitem(
          menu,
          "SMILES -> 2D",
          lambda func:
          coot_gui.generic_single_entry("SMILES string",
                               "", " Send to 2D Viewer ",
                               lambda text: coot.smiles_to_ligand_builder(text)))

    coot_gui.add_simple_coot_menu_menuitem(
          menu,
          "SMILES -> simple 3D",
          lambda func:
          coot_gui.generic_double_entry("Residue name", "SMILES string  ", "LIG", "",
                               False, False, 
                               "Import Molecule",
                               lambda text_1, text_2:
                               coot.import_rdkit_mol_from_smiles(text_1, text_2)))


    coot_gui.add_simple_coot_menu_menuitem(
            menu,
            "Residue -> 2D",
            lambda func:
            coot_utils.using_active_atom(residue_to_ligand_builder,
                              "aa_imol", "aa_chain_id", "aa_res_no",
                              "aa_ins_code", 0.015))

    if coot.enhanced_ligand_coot_p():
        def flev_rdkit_func():
            with coot_utils.UsingActiveAtom() as [aa_imol, aa_chain_id, aa_res_no,
                                       aa_ins_code, aa_atom_name, aa_alt_conf]:
                prodrg_import.fle_view_with_rdkit(aa_imol, aa_chain_id, aa_res_no,
                                    aa_ins_code, 4.2)
                coot.set_flev_idle_ligand_interactions(1)
            
        coot_gui.add_simple_coot_menu_menuitem(
            menu,
            "FLEV this residue",
            lambda func: flev_rdkit_func()
            )


        coot_gui.add_simple_coot_menu_menuitem(
            menu,
            "Toggle FLEV Ligand Interactions",
            lambda func: toggle_idle_ligand_interactions()
            )

        def go_solid_func(state):
            set_display_generic_objects_as_solid(state)
            graphics_draw()

        add_simple_coot_menu_menuitem(
             menu, "Solid Generic Objects",
             lambda func: go_solid_func(1))

        add_simple_coot_menu_menuitem(
             menu, "Unsolid Generic Objects",
             lambda func: go_solid_func(0))


        def show_chem_func():
            set_display_generic_objects_as_solid(1) # there may be consequences...
            with coot_utils.UsingActiveAtom() as [aa_imol, aa_chain_id, aa_res_no,
                                       aa_ins_code, aa_atom_name, aa_alt_conf]:
                coot.show_feats(aa_imol, aa_chain_id, aa_res_no, aa_ins_code)
            
        coot_gui.add_simple_coot_menu_menuitem(
            menu,
            "Show Chemical Features",
            lambda func: show_chem_func()
            )

    def rename_atoms_to_reference(menuitem):
        with coot_utils.UsingActiveAtom() as [aa_imol, aa_chain_id, aa_res_no,
                                   aa_ins_code, aa_atom_name, aa_alt_conf]:
           # user chooses cif_dict_file_name_out (provide default)
           # reference_comp_id and new_comp_id

           # args: entry_info_list, check_button_info, go_button_label, handle_go_function
           coot_gui.generic_multiple_entries_with_check_button(
               [["      Reference Residue Type: ", "ATP"],
                ["               New Residue Type: ", "RXC"],
                ["New Dictionary cif file name: ", "new-dictionary-RXC.cif"]],
               False,
               "   OK   ",
               lambda ref_comp_id_txt_new_comp_id_txt_new_cif_file_name_txt: coot.match_this_residue_and_dictionary(aa_imol, aa_chain_id, aa_res_no, aa_ins_code, ref_comp_id_txt_new_comp_id_txt_new_cif_file_name_txt[2], ref_comp_id_txt_new_comp_id_txt_new_cif_file_name_txt[0], ref_comp_id_txt_new_comp_id_txt_new_cif_file_name_txt[1]))
        

    coot_gui.add_simple_coot_menu_menuitem(
        menu, "Rename Atom to Reference", rename_atoms_to_reference)

    def go_solid_func(state):
        set_display_generic_objects_as_solid(state)
        coot.graphics_draw()

    coot_gui.add_simple_coot_menu_menuitem(
         menu, "Solid Generic Objects",
         lambda func: go_solid_func(1))

    coot_gui.add_simple_coot_menu_menuitem(
         menu, "Unsolid Generic Objects",
         lambda func: go_solid_func(0))


    def tab_ligand_distortions_func():
        with coot_utils.UsingActiveAtom() as [aa_imol, aa_chain_id, aa_res_no,
                                   aa_ins_code, aa_atom_name, aa_alt_conf]:
             coot.print_residue_distortions(aa_imol, aa_chain_id, aa_res_no, aa_ins_code)

    def display_ligand_distortions_func():
         with coot_utils.UsingActiveAtom() as [aa_imol, aa_chain_id, aa_res_no,
                                    aa_ins_code, aa_atom_name, aa_alt_conf]:
             set_display_generic_objects_as_solid(1)
             coot.display_residue_distortions(aa_imol, aa_chain_id, aa_res_no, aa_ins_code)

    coot_gui.add_simple_coot_menu_menuitem(
         menu,
         "Tabulate (on terminal) Ligand Distorsions",
         lambda func: tab_ligand_distortions_func()
         )

    coot_gui.add_simple_coot_menu_menuitem(
         menu,
         "Display Ligand Distortions",
         lambda func: display_ligand_distortions_func())

    add_simple_coot_menu_menuitem(
        menu,
        "Quick Ligand Validate",
        lambda func: gui_ligand_check_dialog_active_residue())


##        # not interesting for the normal user!?
##        def density_ligand_score_func():
##            with coot_utils.UsingActiveAtom() as [aa_imol, aa_chain_id, aa_res_no,
##                                       aa_ins_code, aa_atom_name, aa_alt_conf]:
##               spec = [aa_chain_id, aa_res_no, aa_ins_code]
##               r = coot.density_score_residue(aa_imol, spec, coot.imol_refinement_map())
##               # BL says:: maybe a dialog for this?!
##               print "density at ligand atoms:", r

##        coot_gui.add_simple_coot_menu_menuitem(
##            menu,
##            "Density Score Ligand",
##            lambda func: density_ligand_score_func()
##            )


##        def fetch_ligand_pdbe_func():
##            with coot_utils.UsingActiveAtom() as [aa_imol, aa_chain_id, aa_res_no,
##                               aa_ins_code, aa_atom_name, aa_alt_conf]:
##               comp_id = coot.residue_name(aa_imol, aa_chain_id, aa_res_no, aa_ins_code)
##               print "here with residue name", comp_id
##               s = coot_utils.get_SMILES_for_comp_id_from_pdbe(comp_id)
##               if (isinstance(s, str)):
##                   pdbe_cif_file_name = os.path.join("coot-download", "PDBe-" + comp_id + ".cif")
##                   prodrg_import.import_from_3d_generator_from_mdl(pdbe_cif_file_name, comp_id)

##        coot_gui.add_simple_coot_menu_menuitem(
##            menu,
##            "### [Fetch ligand description & generate restraints]",
##            lambda func: fetch_ligand_pdbe_func()
##            )


    def probe_ligand_func():
        global probe_command
        with coot_utils.UsingActiveAtom() as [aa_imol, aa_chain_id, aa_res_no,
                            aa_ins_code, aa_atom_name, aa_alt_conf]:
            ss = "//" + aa_chain_id + "/" + str(aa_res_no)
            imol_selection = coot.new_molecule_by_atom_selection(aa_imol, ss)
            work_dir = coot_utils.get_directory("coot-molprobity")
            tmp_selected_ligand_for_probe_pdb = os.path.join(work_dir,
                                                "tmp-selected-ligand-for-probe.pdb")
            tmp_protein_for_probe_pdb = os.path.join(work_dir,
                                                 "tmp-protein-for-probe.pdb")
            generic_objects.probe_dots_file_name = os.path.join(work_dir, "probe.dots")

            coot.set_mol_displayed(imol_selection, 0)
            coot.set_mol_active(imol_selection, 0)
            # BL comment: we assume H and view is correct.
            #set_go_to_atom_molecule(imol)
            #rc = residue_centre(imol, chain_id, res_no, ins_code)
            #set_rotation_centre(*rc)
            #hydrogenate_region(6)
            coot.write_pdb_file(imol_selection, tmp_selected_ligand_for_probe_pdb)
            coot.write_pdb_file(imol, tmp_protein_for_probe_pdb)
            coot_utils.popen_command(probe_command, ["-u", "-once", str(aa_res_no), # -once or -both
                                             "not " + str(aa_res_no),
                                             "-density60",
                                             tmp_selected_ligand_for_probe_pdb,
                                             tmp_protein_for_probe_pdb],
                                             [], generic_objects.probe_dots_file_name, False)
            coot.handle_read_draw_probe_dots_unformatted(dots_file_name, aa_imol, 0)
            coot.graphics_draw()






#!/usr/bin/env python
"""Simple statistics functions"""

# Copyright 2010 Kevin Keating
# 
# Licensed under the Educational Community License, Version 2.0 (the
# "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at
# 
# http://www.osedu.org/licenses/ECL-2.0
# 
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS"
# BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
# or implied. See the License for the specific language governing
# permissions and limitations under the License.

def median(values):
    """Calculate the median
    
    ARGUMENTS:
        values - a list of values (does not have to be sorted)
    RETURNS:
        the median
    """
    
    values = sorted(values)
    
    length=len(values)
    
    if length % 2:
        return values[(length-1)//2]
    else:
        lower = values[length//2-1]
        upper = values[length//2]
        return float(lower+upper)/2


def lowerQuartile(values):
    """Calculate the lower quartile 
    
    ARGUMENTS:
        values - a list of values (does not have to be sorted)
    RETURNS:
        the lower quartile
    """
    
    values = sorted(values)
    
    length=len(values)
    
    if length % 2:
        return median(values[0:(length-1)//2])
    else:
        return median(values[0:(length)//2])




# This happens when user clicks on the "Launch JLigand" button.
# It starts a jligand and puts it in the background.
#
def launch_jligand_function():

    global jligand_jar
    global jligand_home_env
    global java_command
    
    jligand.start_jligand_listener()
    # maybe this should rather check PATH or similar!? FIXME
    if not os.path.isfile(jligand_jar):

        # Boo.  Give us a warning dialog
        #
        s = "jligand java jar file: " + jligand_jar + " not found"

        # make an extra message telling us that JLIGAND_HOME is
        # not set if it is not set.
        env_message = "Environment variable JLIGAND_HOME not set\n\n" \
                      if not jligand_home_env else ""
        coot.info_dialog(env_message + s)

    else:
        # OK, it does exist - run it!
        #
        java_exe = coot_utils.find_exe(java_command)
        if not java_exe:
            print("BL INFO:: no java found")
        else:
            # first check if we can run it with coot, i.e. is '-version'
            # a valid command line arg
            jligand_version = ["-jar", jligand_jar, "-version"]
            cmd = java_exe + " " + \
                  coot_utils.string_append_with_spaces(jligand_version)
            res = coot_utils.shell_command_to_string(cmd)
            if (not res):
                message = "Sorry, your JLigand:\n\n " + jligand_jar + "\n\n" + \
                          "is not new enough to work with Coot!\n" + \
                          "Please download a new one!"
                coot.info_dialog(message)
            else:
                coot_utils.run_concurrently(java_exe, jligand_args)
                # beam in a new menu to the menu bar:
                if True:
                    if coot_gui_api.main_menubar():
                        jligand_menu = coot_gui.coot_menubar_menu("JLigand")
                        coot_gui.add_simple_coot_menu_menuitem(
                            jligand_menu, "Send Link to JLigand (click 2 monomers)",
                            lambda func: click_select_residues_for_jligand()
                            )

# This happens when user clicks on the "Select Residues for JLigand"
# (or some such) button.  It expects the user to click on atoms of
# the two residues involved in the link.
#
def click_select_residues_for_jligand():

    global imol_jligand_link
    
    def link_em(*args):
        print("we received these clicks", args)
        if (len(args) == 2):
            click_1 = args[0]
            click_2 = args[1]
            print("click_1:", click_1)
            print("click_2:", click_2)
            if ((len(click_1) == 7)
                and (len(click_2) ==7)):
                resname_1 = coot.residue_name(click_1[1],
                                         click_1[2],
                                         click_1[3],
                                         click_1[4])
                resname_2 = coot.residue_name(click_2[1],
                                         click_2[2],
                                         click_2[3],
                                         click_2[4])
                imol_click_1 = click_1[1]
                imol_click_2 = click_2[1]
                chain_click_1 = click_1[2]
                chain_click_2 = click_2[2]
                resno_click_1 = click_1[3]
                resno_click_2 = click_2[3]
                if not (isinstance(resname_1, str) and
                        isinstance(resname_2, str)):
                    print("Bad resnames: %s and %s" %(resname_1, resname_2))
                else:
                    if not (imol_click_1 == imol_click_2):
                        msg = "Two different molecules %s and %s selected.\n" \
                              %(imol_click_1, imol_click_2) + \
                              "Make sure to select residues in the same molecule."
                        coot.info_dialog(msg)
                        imol_jligand_link = False
                    elif (chain_click_1 == chain_click_2 and
                          resno_click_1 == resno_click_2):
                        msg = "Same residue %s %s selected.\n" \
                              %(chain_click_1, resno_click_1) + \
                              "Make sure to select different residues."
                        coot.info_dialog(msg)
                        imol_jligand_link = False
                    else:
                        # happy path
                        imol_jligand_link = imol_click_1
                        jligand.write_file_for_jligand(jligand.click2res_spec(click_1), resname_1,
                                               jligand.click2res_spec(click_2), resname_2)

    user_defined_click(2, link_em)



import coot
import coot_utils
import coot_contact_score_isolated_ligand

def get_metrics_for_ligand(imol, chain_id, res_no, ins_code,
                           refmac_input_mtz_file_name,
                           fobs_col, sig_fobs_col, rfree_col,
                           refmac_dir):

    """
    Args:
        refmac_dir: the dir where input/output refmac files should be written.

    Return:
        [correlation, mogul, bumps, difference_map_stats, b_factor_info]
        where:
            mogul: is either a list [mogul_z_worst, mogul_out_file_name] or an
            error status
            difference_map_stats:
                the output of map_to_model_correlation_stats_py
                [correlation, var_x, var_y, n, sum_x, sum_y, D, D2 (based on mean/sd of the map at the ligand)
                map_mean map_mean_at_ligand map_sd map_sd_at_ligand )
            b_factor_info:
                [median_ratio median_ligand mediand_-env ks_testd_-result]
    """

    def local_refmac(stub_name):

        ligand_spec = [chain_id, res_no, ins_code]
        neighbs = coot.residues_near_residue_py(imol, ligand_spec, 4)

        rn = coot.residue_name(imol, chain_id, res_no, ins_code)
        n_ligand_atoms = coot.het_group_n_atoms(rn)

        if (not coot_utils.isNumber(n_ligand_atoms)):
            print("BL ERROR:: failed liagnd atoms not a number.")
            return False
        else:
            refmac_out_sfs_file_name = os.path.join(refmac_dir, \
                                                    stub_name + "-with-ligand-refmac.mtz")
            with_ligand_pdb_file_name = os.path.join(refmac_dir, \
                                                     stub_name + "-with-ligand.pdb")

            coot.make_directory_maybe(refmac_dir)
            coot.make_directory_maybe("coot-refmac") # XYZOUT goes here
            coot.write_pdb_file(imol, with_ligand_pdb_file_name)
            r = get_recent_pdbe.refmac_calc_sfs_make_mtz_with_columns(with_ligand_pdb_file_name,
                                                      refmac_input_mtz_file_name,
                                                      refmac_out_sfs_file_name,
                                                      fobs_col, sig_fobs_col, rfree_col)
            if not r:
                print("BL ERROR:: failed calculating sfs in refmac")
                return False
            else:
                # happy path
                return refmac_out_sfs_file_name

    # get_correlation at the ligand for the direct (FWT) map
    #
    def get_correlation(stub_name):

        global refmac_extra_params

        refmac_extra_params = ["refine exclude all from " + \
                               str(res_no) + " " + \
                               chain_id + " to " + \
                               str(res_no) + " " + \
                               chain_id]

        print("BL DEBUG:: in get_correlation(): refmac_extra_params:", refmac_extra_params)

        ligand_spec = [chain_id, res_no, ins_code]
        refmac_out_sfs_file_name = local_refmac(stub_name)

        if not refmac_out_sfs_file_name:
            return False
        else:
            # happy path
            imol_map = coot.make_and_draw_map(refmac_out_sfs_file_name, "FWT", "PHWT", "", 0, 0)
            neighbs = coot.residues_near_residue_py(imol, ligand_spec, 4)

            c = coot.map_to_model_correlation(imol, [ligand_spec], neighbs, 0, imol_map)
            coot.close_molecule(imol_map)
            return c

    # return False or a list of stats.
    #
    def get_ligand_difference_map_stats(stub_name):

        global refmac_extra_params

        refmac_extra_params = False # reset - no ligand exclusion

        refmac_out_sfs_file_name = local_refmac(stub_name + "-for-ligand-diff-map")
        ligand_spec = [chain_id, res_no, ins_code]
        neighbs = coot.residues_near_residue_py(imol, ligand_spec, 4)

        if not refmac_out_sfs_file_name:
            return False
        else:
            # happy path
            imol_map = coot.make_and_draw_map(refmac_out_sfs_file_name,
                                         "DELFWT", "PHDELWT", "", 0, 1)
            # now do some stats on the map at the ligand site

            c = coot.map_to_model_correlation_stats_py(imol, [ligand_spec],
                                                  neighbs, 10, imol_map)
            print("BL INFO:: residue %s density statistics %s!" \
                  %(ligand_spec, c))
            return c

    # Return an error status (False, i.e. not a list) or a list
    # [mogul_z_worst, mogul_out_file_name] on success.
    #
    # (PE note: we want the filename so that we can show mogul markup
    # on the ligand when we activate this function from the gui)
    def get_mogul_score(use_cache_qm):

        # if run_result is a list it contains the mogul-output file name
        # if it is False something went wrong
        #
        run_result = run_mogul("BONDS_AND_ANGLES", imol, chain_id,
                               res_no, ins_code,
                               "ligand-check", use_cache_qm)

        print("BL DEBUG:: run_results (mogul): ", chain_id, res_no, run_result)

        if not run_result:
            return False
        else:
            # happy path
            if use_cache_qm:
                # return a random mogul_score
                return [123, run_result]
            mogul_results_list = coot.mogul_results_py(run_result)
            if (not isinstance(mogul_results_list, list)):
                return False
            else:
                if (len(mogul_results_list) == 0):
                    mogul_score = "MOGUL_NO_STATS"
                else:
                    mogul_score = max(mogul_results_list)
                return [mogul_score, run_result]

    def get_ligand_dictionary_based_geometry_stats():
        ligand_spec = [chain_id, res_no, ins_code]
        summary_info = get_ligand_distortion_summary_info(imol, ligand_spec)
        print("##### we got summary-info", summary_info)
        return summary_info

    # return a list: n_bad_overlaps n_hydrogen_bonds n_small_overlaps n_close_contacts n_wide_contacts
    #
    def get_bump_score():
        ligand_spec = [chain_id, res_no, ins_code]
        cs = coot_contact_score_isolated_ligand.contact_score_ligand(imol, ligand_spec)
        coot.graphics_draw()
        return cs

    # return a list of [median_ratio, median_ligand, mediand_-env, ks_test_result]
    # stub_name is only passed so that we can write diagnostics to standard-out.
    #
    def get_b_factor_distribution_metrics(stub_name):

        def median_ratio(ligand_b_factors, env_b_factors):
            # maybe the second one should include the ligand Bs?!
            return median(ligand_b_factors) / median(env_b_factors)

        def filter_out_waters(imol, env_residues):
            def is_not_water(residue_item):
                rn = coot.residue_name(imol,
                                  res_spec_utils.residue_spec_to_chain_id(residue_item),
                                  res_spec_utils.residue_spec_to_res_no(residue_item),
                                  coot_utils.residue_spec_to_ins_code(residue_item))
                return (rn != "HOH" and rn != "WAT")
            return [res_item for res_item in env_residues if is_not_water(res_item)]

        # from http://stackoverflow.com/questions/24101524/finding-median-of-list-in-python
        def median(lst):
            lst = sorted(lst)
            if len(lst) < 1:
                return None
            if len(lst) %2 == 1:
                return lst[((len(lst)+1)//2)-1]
            else:
                return float(sum(lst[(len(lst)//2)-1:(len(lst)//2)+1]))/2.0

        # Return a list of length 2: 
        # The first item is a list of atoms for the residue specified by ligand-spec
        # It may be False, in which case this function failed to return a result
        # If it is not False, the 2nd is a list of atom b-factors
        #
        def ligand_environment_temperature_factors(imol, ligand_spec, radius):
            atoms = residue_info(imol,
                                 res_spec_utils.residue_spec_to_chain_id(ligand_spec),
                                 res_spec_utils.residue_spec_to_res_no(ligand_spec),
                                 coot_utils.residue_spec_to_ins_code(ligand_spec))
            env_residues = coot.residues_near_residue_py(imol, ligand_spec, radius)
            non_water_env_residues = filter_out_waters(imol, env_residues)
            env_residues = [residue_info(imol,
                                         res_spec_utils.residue_spec_to_chain_id(res_spec),
                                         res_spec_utils.residue_spec_to_res_no(res_spec),
                                         coot_utils.residue_spec_to_ins_code(res_spec)) for res_spec in non_water_env_residues]
            # this is a list of residue info not atoms, so flatten
            env_atoms = []
            list(map(env_atoms.extend, env_residues))
            if isinstance(atoms, list):
                r1 = [atom[1][1][0] \
                         if isinstance(atom[1][1], list) \
                         else atom[1][1] for atom in atoms]
            else:
                r1 = False
            if env_atoms:
                r2 = [atom[1][1][0] \
                         if isinstance(atom[1][1], list) \
                         else atom[1][1] for atom in env_atoms]
            else:
                r2 = False

            return [r1, r2]

        # main line of get_b_factor_distribution_metrics
        #
        ligand_spec = [chain_id, res_no, ins_code]
        lig_env_temp_factors = ligand_environment_temperature_factors(imol, ligand_spec, 5)
        if not isinstance(lig_env_temp_factors, list):
            print("Ligand env temp factors not a list\n")
            return False
        if len(lig_env_temp_factors[1]) == 0:
            print("WARNING:: No values in Ligand env temp factors\n")
            return False

        v1 = lig_env_temp_factors[0]
        v2 = lig_env_temp_factors[1]
        print("b-factor kolmogorov-smirnov lig:", stub_name, ligand_spec, v1)
        print("b-factor kolmogorov-smirnov env:", stub_name, ligand_spec, v2)
        temp_factor_median_ratio = median_ratio(v1, v2)
        kolmogorov_smirnov_result = kolmogorov_smirnov(v1, v2)

        return [temp_factor_median_ratio, median(v1), median(v2),
            kolmogorov_smirnov]

    # main line of get_metrics_for_ligand
    #
    stub_name = molecule_name_stub(imol, 0)

    b_factor_info = get_b_factor_distribution_metrics(stub_name)

    print("DEBUG:: ####################### b-factor-info:", b_factor_info)

    # add error checking to this (maybe more?!)
    #
    cor = get_correlation(stub_name)
    if (coot_utils.isNumber(cor)):
        dms = get_ligand_difference_map_stats(stub_name)
        if (not isinstance(dms, list)):
            return False # error
        else:
            mog = get_mogul_score(False)  # use cache for testing only
            if mog:
                bmp = get_bump_score()
                if (isinstance(bmp, list)):
                    return [cor, mog, bmp, dms, b_factor_info]
                else:
                    return False
            else:
                return False
    else:
        return False

# remove residues that are waters from env-residues
#
def filter_out_waters(imol, env_residues):
    def is_not_water(residue_item):
        rn = residue_name(imol,
                          residue_spec_to_chain_id(residue_item),
                          residue_spec_to_res_no(residue_item),
                          residue_spec_to_ins_code(residue_item))
        return (rn != "HOH" and rn != "WAT")
    return [res_item for res_item in env_residues if is_not_water(res_item)]

# the Yes/No tick/cross dialog
#
def gui_ligand_check_dialog_wrapper(imol, imol_map, ligand_spec):

    neighbs = []
    correl = coot.map_to_model_correlation_py(imol, [ligand_spec], neighbs, 0, imol_map)
    cs = coot_contact_score_isolated_ligand.contact_score_ligand(imol, ligand_spec)
    n_bumps = -1
    if cs:
        n_bumps = cs[0]
    geom_dist_max = 1.1
    ligand_metrics = [correl, geom_dist_max, n_bumps]
    percentile_limit = 0.5 # it's a fraction
    coot.gui_ligand_metrics_py(ligand_spec, ligand_metrics, percentile_limit)

# the Yes/No tick/cross dialog
def gui_ligand_check_dialog_active_residue():
    with coot_utils.UsingActiveAtom(True) as [aa_imol, aa_chain_id, aa_res_no,
                                              aa_ins_code, aa_atom_name, aa_alt_conf, aa_res_spec]:
        gui_ligand_check_dialog_wrapper(aa_imol, coot.imol_refinement_map(), aa_res_spec)

def run_mogul(mode, imol, chain_id, res_no, ins_code, prefix_str, use_cache_qm):
    # dummy since I cannot test mogul
    # will return bogus info for now.
    if use_cache_qm:
        # for testing return a random Z and random file name
        return [2, "dummy"]
    else:
        return False




# 02_shelx.py
# Copyright 2008 by The University of York
# Author: Bernhard Lohkamp
# Copyright 2007, 2008 by The University of Oxford
# Author: Paul Emsley

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or (at
# your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA

import os
import unittest
import coot
import shelx
import coot_utils
import coot_testing_utils

# NOTE: hof.fcf has  _refln_A_calc and _refln_B_calc, not fcalc and phase.
# This will not work.  Why is it here?
#
hof_fcf = os.path.join(coot_testing_utils.unittest_data_dir, "hof.fcf")
hof_res = os.path.join(coot_testing_utils.unittest_data_dir, "HOF.RES")

#global imol_hof_res
imol_hof_res = False

insulin_fcf = os.path.join(coot_testing_utils.unittest_data_dir, "insulin.fcf")
insulin_res = os.path.join(coot_testing_utils.unittest_data_dir, "insulin.res")
hollander_ins = os.path.join(coot_testing_utils.unittest_data_dir, "hollander.ins")
global imol_insulin_res
global imol_insulin_map
imol_insulin_res = -1 # see later
imol_insulin_map = -1 # see later
m_miller_res = os.path.join(coot_testing_utils.unittest_data_dir, "miller/shelx-test4-NPD-mini.res")


class TestShelxFunctions(unittest.TestCase):

    def test01_0(self):
        """Read small molecule .res file"""

        imol = coot.read_pdb(hof_res)
        self.assertTrue(coot_utils.valid_model_molecule_qm(imol))
        imol_hof_res = imol


    def test02_0(self):
        """Read hollander small molecule .res file"""

        imol = coot.read_pdb(hollander_ins)
        self.assertTrue(coot_utils.valid_model_molecule_qm(imol), "   fail: bad molecule for %s" %hollander_ins)
        spg = coot.show_spacegroup(imol)
        self.assertEqual(spg, "I 41 2 2", "   fail: wrong spacegroup for %s %s" %(hollander_ins, spg))


    def test03_0(self):
        """read shelx insulin with fcf"""

        global imol_insulin_res
        global imol_insulin_map

        imol_insulin_res_local = coot.handle_read_draw_molecule_with_recentre(insulin_res, 1)
        self.assertTrue(coot_utils.valid_model_molecule_qm(imol_insulin_res_local),
                        "   Bad insulin.res: %s for %s" %(insulin_res, imol_insulin_res_local))

        imol_insulin_res = imol_insulin_res_local  # used in water addition test
        imol = shelx.handle_shelx_fcf_file(insulin_fcf)
        self.assertTrue(coot_utils.valid_map_molecule_qm(imol), "    Bad read of %s %s" %(insulin_fcf, imol))

        # remove the name test for Sigmaa now that we use read_small_molecule_data_cif

        self.assertEqual(coot.show_spacegroup(imol),
                             coot.show_spacegroup(imol_insulin_res_local),
                             "   Mismatch spacegroups %s %s" %(coot.show_spacegroup(imol),
                                                               coot.show_spacegroup(imol_insulin_res_local)))

        self.assertEqual(coot.show_spacegroup(imol), "I 21 3",
                             "   Bad spacegroups %s" %(coot.show_spacegroup(imol)))

        # good then
        imol_insulin_map = imol
        # coot.rotate_y_scene(coot.rotate_n_frames(200), 0.1)
        # close_molecule(imol)    # needed later
        # imol_insulin_res_local needed later -> global?

    # The "SHELX Woe" problem
    #
    def test04_0(self):
        """Write an INS from PDB test"""

        # First, check return status on a bogus molecule
        status = coot.write_shelx_ins_file(204050, "no_molecule.ins")
        self.assertEqual(status, 0, "bad exit status from write_shelx_ins_file on bogus molecule %s" %status)

        # Now check return status on a bogus molecule.  The Happy Path.
        #
        imol = coot_testing_utils.unittest_pdb("tutorial-modern.pdb")
        self.assertTrue(coot_utils.valid_model_molecule_qm(imol), "   imol_rnase not valid.")
        rnase_ins = "rnase.ins"
        status = coot.write_shelx_ins_file(imol, rnase_ins)
        self.assertEqual(status, 1, "   failure to write INS file %s from PDB: status %s" %(rnase_ins, status))


    def test05_0(self):
        """new molecule by atom selection inherits shelx molecule flag"""

        global imol_insulin_res
        insulin_frag = coot.new_molecule_by_atom_selection(imol_insulin_res, "//B/2010-2020")
        self.assertTrue(coot_utils.valid_model_molecule_qm(insulin_frag),
                        " bad fragment of insulin res molecule")
        self.assertTrue(coot_utils.shelx_molecule_qm(insulin_frag),
                        " bad shelx flag from insulin-frag")


    def test06_0(self):
        """Addition of Terminal Residue on SHELX molecule has correct occupancy"""

        global imol_insulin_res
        global imol_insulin_map
        insulin_frag = coot.new_molecule_by_atom_selection(imol_insulin_res, "//B/2010-2020")
        self.assertTrue(coot_utils.valid_model_molecule_qm(insulin_frag),
                        " bad fragment of insulin res molecule")

        coot.set_imol_refinement_map(imol_insulin_map)
        coot.add_terminal_residue(insulin_frag, "B", 2020, "ALA", 1)

        res_atoms = coot.residue_info_py(insulin_frag, "B", 2021, "")

        self.assertTrue(res_atoms, " bad residue info after add terminal residue")

        for atom in res_atoms:
            occ = atom[1][0]
            self.assertAlmostEqual(occ, 11.0, 1, " bad occupancides in new residue %s" %atom)


    def test07_0(self):
        """Add water to SHELX molecule"""

        global imol_insulin_res
        self.assertTrue(coot_utils.valid_model_molecule_qm(imol_insulin_res), "   failed to get a valid imol_insulin_res")

        coot.set_pointer_atom_molecule(imol_insulin_res)
        coot.set_rotation_centre(3, -1, 60)
        coot.place_typed_atom_at_pointer("Water")
        # test is to have to occupancy of the new HOH to be 11.0
        coot_testing_utils.shelx_waters_all_good_occ_qm(self, imol_insulin_res)


    # Tobias Beck test
    def test08_0(self):
        """Find Waters for a SHELXL molecule"""

        self.assertTrue(coot_utils.valid_model_molecule_qm(imol_insulin_res),
                        "   failed to get a valid imol_insulin_res")

        n_chains_pre = coot.n_chains(imol_insulin_res)
        coot.find_waters(imol_insulin_map, imol_insulin_res, 0, 0.6, 1)
        n_chains_post = coot.n_chains(imol_insulin_res)
        self.assertTrue(n_chains_pre == n_chains_post,
                        "Find waters on a shelx molecule created a new chain %s %s"
                        %(n_chains_pre, n_chains_post))
        coot_testing_utils.shelx_waters_all_good_occ_qm(self, imol_insulin_res)


# non positive definite anistropic atom (reported by Mitch Miller)
# crash test
    def test09_0(self):
        """NPD Anisotropic Atom [Mitch Miller]"""

        imol_miller = coot.handle_read_draw_molecule_with_recentre(m_miller_res, 1)

        self.assertTrue(coot_utils.valid_model_molecule_qm(imol_miller), "Bad read of miller test molecule")

        coot.set_show_aniso(1)  # crash?
        # coot.rotate_y_scene(coot.rotate_n_frames(100), 0.1)
        coot.close_molecule(imol_miller)
        coot.set_show_aniso(0)

# cheesy test, close the shelx molecules

    def test10_0(self):
        """close shelx molecules"""

        coot.close_molecule(imol_insulin_map)
        coot.close_molecule(imol_insulin_res)

        self.assertFalse(coot_utils.valid_model_molecule_qm(imol_insulin_res),
                    "imol_insulin_res: %s is still valid after closure!"
                    %imol_insulin_res)
        self.assertFalse(coot_utils.valid_map_molecule_qm(imol_insulin_map),
                    "imol_insulin_map: %s is still valid after closure!"
                    %imol_insulin_map)


    def test11_0(self):
        """Aniso Bs in P21"""

        # from types import ListType

        def aniso_b_from_atom(atom):
            occ_etc = atom[1]
            return occ_etc[1]

        imol = coot_testing_utils.unittest_pdb("horma-p21.res")
        self.assertTrue(coot_utils.valid_model_molecule_qm(imol),
                        "  failed to get a valid imoll from horma-p21.res")
        coot.write_shelx_ins_file(imol, "new-horma.ins")
        imol_2 = coot.read_pdb("new-horma.ins")
        at_1 = coot_utils.get_atom(imol   , "A", 4, "", " N1 ")
        at_2 = coot_utils.get_atom(imol_2 , "A", 4, "", " N1 ")
        b_1  = aniso_b_from_atom(at_1)
        b_2  = aniso_b_from_atom(at_2)

        print("b_1:", b_1)
        print("b_2:", b_2)

        list(map(lambda x, y: self.assertAlmostEqual(x, y), b_1, b_2))


    def test12_0(self):
        """Don't crash on reading a strange HAT file"""

        coot.handle_read_draw_molecule_with_recentre(os.path.join(coot_testing_utils.unittest_data_dir, "crash.hat"), 0)
        # this is nonsense since we already 'tested' for read/crash
        # but it is a proper test...
        imol = coot_testing_utils.unittest_pdb("crash.hat")
        self.assertTrue(coot_utils.valid_model_molecule_qm(imol))



# ncs.py
# Copyright 2007, 2008 by Bernhard Lohkamp
# copyright 2008 The University of York
# Copyright 2007 by Paul Emsley, The University of York
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import coot
from coot_redefine_functions import *

def find_first_model_molecule():

    imols = molecule_number_list()
    if not imols:
        return False
    else:
        for molecule in imols:
            if coot_utils.valid_model_molecule_qm(molecule):
                return molecule

# Skip the residue in the next chain (typically of a molecule with
# NCS) with the same residue number.  If on the last chain, then wrap
# to beginning.  If it can't find anything then don't move (and put a
# message in the status bar)
#
def skip_to_next_ncs_chain(direction):

  import types

  # Given a chain-id and a list of chain-ids, return the chain-id of
  # the next chain to be jumped to (use wrapping).  If the list of
  # chain-ids is less then length 2, return False.
  #
  def skip_to_chain_internal(this_chain_id, chain_id_list):
    # print "this_chain_id: ", this_chain_id
    if len(chain_id_list) < 2:
       return False
    else:
        # we do it differnt to Paul's function, as I dont understand it and
        # feel that this is easier and equally good!?
        # Agreed, it does the same thing and more obviously correct 20100126-PE
        current_chain_index = chain_id_list.index(this_chain_id)
        if current_chain_index == len(chain_id_list)-1:	# last chain
            # return the first chain
            return chain_id_list[0]
        else:
            # return the next chain
            return chain_id_list[current_chain_index + 1]


  def skip_to_chain(imol, this_chain_id, chain_id_list):

      # Given a chain-id and a list of chain-ids, return the chain-id of
      # the next chain to be jumped to (use wrapping).  If the list of
      # chain-ids is less then length 2, return False.
      #
      chain_guess = skip_to_chain_internal(this_chain_id, chain_id_list)

      if ((not type(chain_guess) is bytes)):
          return chain_guess
      elif (coot_utils.is_solvent_chain_qm(imol, chain_guess)):
          skip_to_chain(imol, chain_guess, chain_id_list)
      else:
          return chain_guess

  def get_chain_id_list(imol, this_chain_id):
      att = ncs_chain_ids(imol)
      if (not att):
          return coot_utils.chain_ids(imol)
      else:
          for attempt in att:
              if (this_chain_id in attempt):
                  return attempt
          # we havent found this_chain_id in ncs chains, so return all chains
          return coot_utils.chain_ids(imol)

  # First, what is imol? imol is the go to atom molecule
  imol = coot.go_to_atom_molecule_number()
  this_chain_id = coot.go_to_atom_chain_id()
  chains = get_chain_id_list(imol, this_chain_id)
  # try to get ghosts to be able to apply orientations
  coot.make_ncs_ghosts_maybe(imol)
  found_atom_state = 0
  if (not chains):
      print("BL WARNING:: empty set of chains!!! This should never happen.")
      msg_txt = "BL WARNING:: somehow there are no chains in mol " + str(imol)
      msg_txt+= "\n Try \"p\" to update the Go To Atom molecule and then\n"
      msg_txt+= "skip again. Good luck skipping!\n"
      coot.info_dialog(msg_txt)
      # what shall we do now? Bail out I guess. But this should never happen!
      return
  next_chain = skip_to_chain(imol, this_chain_id, chains)

  try_next_chain = next_chain
  while (not try_next_chain == this_chain_id):

      # OK, stop trying for next chain if we have looped round
      # (as it were) so that we are back at the starting chain:
      # e.g. consider the case: ["A" is protein, "B" is water,
      # "C" is ligand]
      #
      if (not try_next_chain):
          coot.add_status_bar_text("No 'NCS Next Chain' found")
          break
      else:
          if (not (try_next_chain == this_chain_id)):
              found_atom_state = coot.set_go_to_atom_chain_residue_atom_name_no_redraw(
                  try_next_chain,
                  coot.go_to_atom_residue_number(),
                  coot.go_to_atom_atom_name(),
                  0)
              
              # now, did that set-go-to-atom function work (was there a
              # real atom)?  If not, then that could have been the ligand
              # chain or the water chain that we tried to go to.  We want
              # to try again, and we shbould keep trying again until we get
              # back to this-chain-id - in which case we have a "No NCS
              # Next Chain atom" status-bar message.

          if (found_atom_state == 0):
              # then we did *not* find the atom, e.g. next-chain was
              # the water chain
              try_next_chain = skip_to_chain(imol, try_next_chain, chains)
          else:
              # otherwise all was hunkey-dorey
              # set the orientation
              forward_flag = 0
              if (direction == "forward"):
                  forward_flag = 1
              coot.apply_ncs_to_view_orientation_and_screen_centre(imol, this_chain_id, next_chain, forward_flag)
              break
  
            

# A function inspired by a question from Bill Scott.  He wanted to
# RNA ghosts.  Because RNA does not work with SSM, we need to define
# the matrix manually.  Let's make a copy of given imol and get
# the rtop from that.  Typical usage manual_ncs_ghosts(0, 1, 10, "A", "C")
# 
def single_manual_ncs_ghosts(imol, resno_start, resno_end, ref_chain, peer_chain):

	imol_copy = coot.copy_molecule(imol)
	clear_lsq_matches()
	add_lsq_match(resno_start, resno_end, ref_chain, 
                      resno_start, resno_end, peer_chain, 0) # ref mov - all atoms
	rtop = apply_lsq_matches(imol_copy, imol_copy)
	close_molecule(imol_copy)
	if (not rtop):
            print("Failed to get matching matrix")
	else:
            coot.clear_ncs_ghost_matrices(imol)
            coot.set_draw_ncs_ghosts(imol, 1)
            args = [imol, peer_chain, ref_chain] + rtop[0] + rtop[1]
            coot.add_ncs_matrix(*args)

# chain-id-list is ["A", "B", "C", "D"], i.e. the
# reference/target/master chain-id first and then the peers.  This
# allows us to add many peers at the same time (unlike above
# function).
#
def manual_ncs_ghosts(imol, resno_start, resno_end, chain_id_list):

    # from types import ListType

    if True:
        if (len(chain_id_list) > 1):
            # OK, OK, to the standard SSM-based NCS matrices are bad for this
            # molecule, lets use LSQ.
            coot.clear_ncs_ghost_matrices(imol)
            imol_copy = coot.copy_molecule(imol)
            for chain_id in chain_id_list[1:]:       # I dont think we need to superpose A onto A?!
                coot.clear_lsq_matches()
                coot.add_lsq_match(resno_start, resno_end, chain_id_list[0],
                              resno_start, resno_end, chain_id, 1)
                rtop = apply_lsq_matches(imol, imol_copy)
                if (not rtop):
                    print("Failed to get LSQ matching matrix", chain_id)
                else:
                    master = chain_id_list[0]
                    print("chain_id %s master %s rtop: %s" %(chain_id, master, rtop))
                    if master:  # should be string or False, so ok I belive
                        args = [imol, chain_id, master] + rtop[0] + rtop[1]
                        coot.add_ncs_matrix(*args)
            coot.close_molecule(imol_copy)
            coot.set_draw_ncs_ghosts(imol, 1)

# Update NCS ghosts based on local environment (residues within 6A of
# (and including) the active residue).
#
# Typically one would bind this function to a key.
#
def update_ncs_ghosts_by_local_sphere():
    """Update NCS ghosts based on local environment (residues within 6A of
    (and including) the active residue).
    
    Typically one would bind this function to a key."""

    with coot_utils.UsingActiveAtom() as [aa_imol, aa_chain_id, aa_res_no, aa_ins_code,
                               aa_atom_name, aa_alt_conf]:
        # coot.clear_ncs_ghost_matrices(aa_imol)
        ghost_ncs_chain_ids = ncs_chain_ids(aa_imol)
        if (isinstance(ghost_ncs_chain_ids, list)):
            ghost_chain_id_list = ghost_ncs_chain_ids[0]
            if (isinstance(ghost_chain_id_list, list)):
                coot.clear_ncs_ghost_matrices(aa_imol)
                for chain_id in ghost_chain_id_list[1]:
                    imol_copy = coot.copy_molecule(aa_imol)
                    coot.set_mol_displayed(imol_copy, 0)
                    coot.set_mol_active(imol_copy, 0)
                    coot.clear_lsq_matches()
                    active_residue_spec = [aa_chain_id, aa_res_no, aa_ins_code]
                    near_residues = coot.residues_near_residue_py(aa_imol, active_residue_spec, 6)
                    sphere_residues = [active_residue_spec, near_residues]
                    for residue_spec in sphere_residues:
                        res_no = res_spec_utils.residue_spec_to_res_no(residue_spec)
                        coot.add_lsq_match(res_no, res_no, ghost_chain_id_list[0],
                                      res_no, res_no, chain_id, 1)

                    rtop = apply_lsq_matches(aa_imol, imol_copy)
                    coot.close_molecule(imol_copy)
                    if not rtop:
                        print("Failed to get LSQ matching matrix", chain_id)
                    else:
                        master = ghost_chain_id_list[0]
                        if (isinstance(master, str)):
                            args = [aa_imol, chain_id, master] + rtop[0] + rtop[1]
                            coot.add_ncs_matrix(*args)

            
# Return the first master chain id (usually there is only one of course) or False
#
def ncs_master_chain_id(imol):

    from types import ListType
    cids_ls = ncs_chain_ids(imol)
    if type(cids_ls) is not ListType:
        return False
    else:
        if (len(cids_ls) == 0):
            return False
        else:
            cids = cids_ls[0]
            if (len(cids) == 0):
                return False
            else:
                return cids[0]

# This was designed to create an NCS copy of a ligand (or range of
# residues) in the active site of one chain to the as yet unoccupied
# active site of another, i.e. it makes a NCS ligand "D"1 that is a NCS
# copy of ligand "C"1 using an NCS operator that maps protein chain "A"
# onto chain "B".
# 
def ncs_ligand(imol_protein, ncs_master_chain_id,
               imol_ligand, chain_id_ligand,
               resno_ligand_start, resno_ligand_stop):
    
    # find ghost in ghosts that has a chain-id matching
    # chain-id-protein and get its rtop.  Return #f on not finding the
    # ghost
    def rtop_from_ghost_with_chain_id(ghosts, chain_id):
        for ghost in ghosts:
            if (ghost[1] == chain_id):
                return ghost[3]
        return False

    coot_utils.chain_ids_from_ncs = ncs_chain_ids(imol_protein)
    if coot_utils.chain_ids_from_ncs:
        ligand_selection_string = "//" + chain_id_ligand + "/" + str(resno_ligand_start) + \
                                  "-" + str(resno_ligand_stop)
        imol_ligand_fragment = coot.new_molecule_by_atom_selection(imol_ligand, ligand_selection_string)
        ghosts = ncs_ghosts(imol_protein)
        for coot_utils.chain_ids in chain_ids_from_ncs:
            if (chain_ids[0] == ncs_master_chain_id):
                peer_chains = coot_utils.chain_ids[1:len(chain_ids)]
                candidate_name = "Candidate NCS-related ligand"
                for chain_id_protein in peer_chains:
                    rtop = rtop_from_ghost_with_chain_id(ghosts, chain_id_protein)
                    if (not rtop):
                        print("Opps - ncs-ligand: Missing ghost rt-op!")
                        coot.info_dialog("Opps - ncs-ligand: Missing ghost rt-op!")
                    else:
                        new_lig_mol = coot.copy_molecule(imol_ligand_fragment)
                        coot_utils.transform_coords_molecule(new_lig_mol, inverse_rtop(rtop))
                        coot.set_molecule_name(new_lig_mol,
                                          str(new_lig_mol) +
                                          ": " +
                                          candidate_name +
                                          " to protein chain " +
                                          chain_id_protein)

        def test_func(imol):
            if (not coot_utils.valid_model_molecule_qm(imol)):
                return False
            else:
                name = coot.molecule_name(imol)
                if (candidate_name in name):
                    ls = [name]
                    for i in coot_utils.molecule_centre(imol):
                        ls.append(i)
                    return ls
                else:
                    return False
        fitting.molecules_matching_criteria(lambda imol: test_func(imol))     
                
                
                



# Copyright 2006 Joel Bard
# Copyright 2006 by Bernhard Lohkamp
# Copyright 2006 by Paul Emsley, The University of York

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or (at
# your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc.,  51 Franklin Street, Fifth Floor, 
# Boston, MA 02110-1301, USA

# Read in cns coeff-data (given filenames) and a pdb molecule filename to make
# maps.
#
def cns2coot(twofofc_coeffs, fofc_coeffs, model_pdb):
    import os, string

    # remove any trailing spaces of string
    # BL says: maybe somthing for coot-utils?
    # ..and has already a buikld-in function in python
    def trim_trailing_spaces(s):
        f = string.rstrip(s)
        return f

    # return [] on bad, else return a 6 memberes list.
    def get_cell_from_pdb(pdb_file):
        s = []
        fin = False
        try:
           fin = open(pdb_file,'r')
        except IOError:
           print("BL WARNING:: Cannot read ", pdb_file)
        if (fin):
           lines = fin.readlines()
           for line in lines:
               if "CRYST1" in line:
                  s = string.split(line)
                  s = s[1:7]
                  fin.close()
                  break
        else:
           print("BL WARNING:: IOError in ", pdb_file) 
        return s

    #  return "" on bad, else return a string that is the space group.
    #   
    #  have to go through all sorts of hoops because sometimes we might
    #  get P 21 and sometimes we might get P 1 21 1 for example
    #  sftools needs P21, can't handle P1211
    #  that being said - there must be a better way
    # set SYMM1=`awk 'BEGIN { FIELDWIDTHS="55 11 4" } /CRYST1/ {gsub(" ","",$2); coot_utils.printf "%s\n", $2}' $3`
    #
    def get_symm_from_pdb_file(pdb_file):

        s = ""
        fin = False
        try:
           fin = open(pdb_file,'r')
        except IOError:
           print("BL WARNING:: Cannot read ", pdb_file)
        if (fin):
           lines = fin.readlines()
           for line in lines: 
               if "CRYST1" in line:
                  s = string.split(line)
                  s = s[7:]
                  s = string.join(s,"")
                  fin.close()
                  break
        else:
           print("BL WARNING:: IOError in ", pdb_file)
        return s

    def get_symm_from_pdb_file_2(pdb_file):

        s = ""
        fin = False
        try:
           fin = open(pdb_file,'r')
        except IOError:
           print("BL WARNING:: Cannot read ", pdb_file)
        if (fin):
           lines = fin.readlines()
           for line in lines:
               if "CRYST1" in line:
                  s = string.split(line)
                  s = s[7:]
                  s = string.join(s)
                  fin.close()
                  break
        else:
           print("BL WARNING:: IOError in ", pdb_file)
        return s

    # Return a symmetry string
    # on problem, return False. Use that to exit (elsewhere).
    #
    def get_symm_from_symop_lib(symm2):
        
        try: 
           e = os.environ['CCP4']
        except:
           print("CCP4 not set up.")
           return False
        if (e):
           s = False
           e = os.path.join(e,"lib","data","symop.lib")
           fin = False
           try:
              fin=open(e,'r')
           except:
              print("Problems opening ", e)
           if (fin):
              lines = fin.readlines()
              for line in lines: 
                 if symm2 in line:
                    t = string.split(line)
                    s = t[3]
                    break
              fin.close()
           return s
        else:
           print("Problems with CCP4 set up")
        
    # main body
    map1_prefix = coot_utils.strip_extension(twofofc_coeffs)
    map2_prefix = coot_utils.strip_extension(fofc_coeffs)
    map1_tmp = map1_prefix + "_tmp.pdb"
    map2_tmp = map2_prefix + "_tmp.pdb"
    cell = get_cell_from_pdb(model_pdb)

    print("map1_prefix: ", map1_prefix)
    print("map2_prefix: ", map2_prefix)
    print("cell: ", cell)

    symm1 = get_symm_from_pdb_file(model_pdb)
    pdbset_log = "cns2coot-pdbset-tmp.log"

    print("symm1: ", symm1)
    coot_utils.popen_command("pdbset",["XYZIN",model_pdb,"XYZOUT",map1_tmp],["CELL " + string.join(cell), "SPACEGROUP " + symm1],pdbset_log,0)
    symm2 = get_symm_from_pdb_file_2(map1_tmp)
    print("symm2: ", symm2)
    symm = get_symm_from_symop_lib(symm2)

    if not(symm):
       print("Failed to find symm in symop.lib!")
    else:
       print("INFO:: SYMM is ", symm)
       map1_mtz = map1_prefix + ".mtz"
       map2_mtz = map2_prefix + ".mtz"
       map_coot_mtz = map1_prefix + "-coot.mtz"
       cad_log = "cns2coot-cad-tmp.log"
       sftools_1_log = "cns2coot-sftools-1-tmp.log"
       sftools_2_log = "cns2coot-sftools-2-tmp.log"

       if os.path.isfile(map1_mtz):
          os.remove(map1_mtz)
       if os.path.isfile(map2_mtz):
          os.remove(map2_mtz)
   
       coot_utils.popen_command("sftools", [] ,["read " + twofofc_coeffs, "cns", string.join(cell), symm, "END", "W", "P", "R", "SET LABELS", "FOM", "PHIC", "SCALE", "FWT", "PHWT", "WRITE " + map1_mtz] , sftools_1_log, 0)

       coot_utils.popen_command("sftools", [] ,["read " + fofc_coeffs, "cns", string.join(cell), symm, "END", "W", "P", "R", "SET LABELS", "FOM", "PHIC", "SCALE", "DELFWT", "PHDELWT", "WRITE " + map2_mtz] , sftools_2_log, 0)

       coot_utils.popen_command("cad", ["HKLIN1", map1_mtz, "HKLIN2", map2_mtz, "HKLOUT", map_coot_mtz],["LABIN FILE_NUMBER 1 E1=FOM E2=PHIC E3=FWT E4=PHWT", "LABIN FILE_NUMBER 2 E1=DELFWT E2=PHDELWT", "END"], cad_log, 0)

       if os.path.isfile(map1_mtz):
          os.remove(map1_mtz)
       if os.path.isfile(map2_mtz):
          os.remove(map2_mtz)

       # now load them in Coot
       #
       coot.read_pdb(model_pdb)
       coot.make_and_draw_map_with_reso_with_refmac_params(map_coot_mtz, "FWT", "PHWT", "", 0, 0, 0, "Fobs:None-specified", "SigF:None-specified", "RFree:None-specified", 0, 0, 0, -1.00, -1.00)
       coot.make_and_draw_map_with_reso_with_refmac_params(map_coot_mtz, "DELFWT", "DELPHWT", "", 0, 1, 0, "Fobs:None-specified", "SigF:None-specified", "RFree:None-specified", 0, 0, 0, -1.00, -1.00)

#cns_coot("2fo-fc_Fum480.coeff","fo-fc_Fum480.coeff","bindividual_Fum480.pdb")




# Define an environment variable for the place where jligand.jar
# resides in a canonical distribution
# 

# should be global?
to_jligand_secret_file_name = ".coot-to-jligand-8lcs"
global from_jligand_secret_file_name
from_jligand_secret_file_name = ".jligand-to-coot"

jligand_home = os.getenv("JLIGAND_HOME") if os.getenv("JLIGAND_HOME") else "."

java_command = "java"  # may have to absolute FIXME
jligand_jar = os.path.join(os.path.normpath(jligand_home), "jligand.jar")
jligand_args = ["-jar", jligand_jar, "-coot"]

def write_file_for_jligand(resname_1, resname_2):

    fin = open(to_jligand_secret_file_name, 'w')
    int_time = time.time()
    fin.write("CODE " + resname_1 + "\n")
    fin.write("CODE " + resname_2 + "\n")
    fin.write("TIME " + str(int_time) + "\n")
    fin.close()

if True:
  if coot_gui_api.main_menubar():
      
      menu = coot_gui.coot_menubar_menu("JLigand")

      def launch_jligand():
          if not os.path.isfile(jligand_jar):
              s = "jligand java jar file: " + jligand_jar + " not found"
              coot.info_dialog(s)
          else:
              java_exe = coot_utils.find_exe(java_command)
              if java_exe:
                  coot_utils.run_concurrently(java_exe, jligand_args)
              else:
                  print "BL INFO:: no java found"
          
      coot_gui.add_simple_coot_menu_menuitem(
          menu, "Launch Jligand",
          lambda func: launch_jligand()
          )


      def make_jligand_link():
          def link_em(*args):
              print "we received these clicks", args
              if (len(args) == 2):
                  click_1 = args[0]
                  click_2 = args[1]
                  if ((len(click_1) == 7)
                      and (len(click_2) ==7)):
                      resname_1 = coot.residue_name(click_1[1],
                                               click_1[2],
                                               click_1[3],
                                               click_1[4])
                      resname_2 = coot.residue_name(click_2[1],
                                               click_2[2],
                                               click_2[3],
                                               click_2[4])
                      if not (isinstance(resname_1, str) and
                              isinstance(resname_2, str)):
                          print "Bad resnames: %s and %s" %(resname_1, resname_2)
                      else:
                          write_file_for_jligand(resname_1, resname_2)
                          
          user_defined_click(2, link_em)
          
      coot_gui.add_simple_coot_menu_menuitem(
          menu, "Send Link to JLigand (click 2 monomers)",
          lambda func: make_jligand_link()
          )

def handle_read_from_jligand_file():
    global from_jligand_secret_file_name
    if (os.path.isfile(from_jligand_secret_file_name)):
        coot.read_cif_dictionary(from_jligand_secret_file_name)

global startup_mtime
startup_mtime = prodrg_import.get_file_latest_time(from_jligand_secret_file_name)

def jligand_timeout_func():
    
    global startup_mtime
    global from_jligand_secret_file_name

    import operator
    
    now_time = prodrg_import.get_file_latest_time(from_jligand_secret_file_name)
    #print "startup-mtime: %s   now-time: %s" %(startup_mtime, now_time)
    if operator.isNumberType(now_time):
        if not operator.isNumberType(startup_mtime):
            startup_mtime = now_time
            print "just set startup-mtime to (first)", startup_mtime
            handle_read_from_jligand_file()
        else:
            if (now_time > startup_mtime):
                startup_mtime = now_time
                print "just set startup-mtime to", startup_mtime
                handle_read_from_jligand_file()
                
    return True # we never expire...
                
gobject.timeout_add(500, jligand_timeout_func)
